import datetime
import logging

from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.contrib.auth.models import Group
from django.shortcuts import render, redirect

from allianceauth.services.forms import ServicePasswordForm
from .forms import JabberBroadcastForm
from .manager import OpenfireManager, PingBotException
from .models import OpenfireUser
from .tasks import OpenfireTasks

logger = logging.getLogger(__name__)

ACCESS_PERM = 'openfire.access_openfire'


@login_required
@permission_required(ACCESS_PERM)
def activate_jabber(request):
    logger.debug("activate_jabber called by user %s" % request.user)
    character = request.user.profile.main_character
    logger.debug("Adding jabber user for user %s with main character %s" % (request.user, character))
    info = OpenfireManager.add_user(OpenfireTasks.get_username(request.user))
    # If our username is blank means we already had a user
    if info[0] is not "":
        OpenfireUser.objects.update_or_create(user=request.user, defaults={'username': info[0]})
        logger.debug("Updated authserviceinfo for user %s with jabber credentials. Updating groups." % request.user)
        OpenfireTasks.update_groups.delay(request.user.pk)
        logger.info("Successfully activated jabber for user %s" % request.user)
        messages.success(request, 'Activated jabber account.')
        credentials = {
            'username': info[0],
            'password': info[1],
        }
        return render(request, 'services/service_credentials.html',
                      context={'credentials': credentials, 'service': 'Jabber'})
    else:
        logger.error("Unsuccessful attempt to activate jabber for user %s" % request.user)
        messages.error(request, 'An error occurred while processing your jabber account.')
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def deactivate_jabber(request):
    logger.debug("deactivate_jabber called by user %s" % request.user)
    if OpenfireTasks.has_account(request.user) and OpenfireTasks.delete_user(request.user):
        logger.info("Successfully deactivated jabber for user %s" % request.user)
        messages.success(request, 'Deactivated jabber account.')
    else:
        logger.error("Unsuccessful attempt to deactivate jabber for user %s" % request.user)
        messages.error(request, 'An error occurred while processing your jabber account.')
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def reset_jabber_password(request):
    logger.debug("reset_jabber_password called by user %s" % request.user)
    if OpenfireTasks.has_account(request.user):
        result = OpenfireManager.update_user_pass(request.user.openfire.username)
        # If our username is blank means we failed
        if result != "":
            logger.info("Successfully reset jabber password for user %s" % request.user)
            messages.success(request, 'Reset jabber password.')
            credentials = {
                'username': request.user.openfire.username,
                'password': result,
            }
            return render(request, 'services/service_credentials.html',
                          context={'credentials': credentials, 'service': 'Jabber'})
    logger.error("Unsuccessful attempt to reset jabber for user %s" % request.user)
    messages.error(request, 'An error occurred while processing your jabber account.')
    return redirect("services:services")


@login_required
@permission_required('auth.jabber_broadcast')
def jabber_broadcast_view(request):
    logger.debug("jabber_broadcast_view called by user %s" % request.user)
    allchoices = []
    if request.user.has_perm('auth.jabber_broadcast_all'):
        allchoices.append(('all', 'all'))
        for g in Group.objects.all():
            allchoices.append((str(g.name), str(g.name)))
    else:
        for g in request.user.groups.all():
            allchoices.append((str(g.name), str(g.name)))
    if request.method == 'POST':
        form = JabberBroadcastForm(request.POST)
        form.fields['group'].choices = allchoices
        logger.debug("Received POST request containing form, valid: %s" % form.is_valid())
        if form.is_valid():
            main_char = request.user.profile.main_character
            logger.debug("Processing jabber broadcast for user %s with main character %s" % (request.user, main_char))
            try:
                if main_char is not None:
                    message_to_send = form.cleaned_data[
                                          'message'] + "\n##### SENT BY: " + "[" + main_char.corporation_ticker + "]" + \
                                      main_char.character_name + " TO: " + \
                                      form.cleaned_data['group'] + " WHEN: " + datetime.datetime.utcnow().strftime(
                        "%Y-%m-%d %H:%M:%S") + " #####\n##### Replies are NOT monitored #####\n"
                    group_to_send = form.cleaned_data['group']

                else:
                    message_to_send = form.cleaned_data[
                        'message'] + "\n##### SENT BY: " + "No character but can send pings?" + " TO: " + \
                        form.cleaned_data['group'] + " WHEN: " + datetime.datetime.utcnow().strftime(
                        "%Y-%m-%d %H:%M:%S") + " #####\n##### Replies are NOT monitored #####\n"
                    group_to_send = form.cleaned_data['group']

                OpenfireManager.send_broadcast_message(group_to_send, message_to_send)

                messages.success(request, 'Sent jabber broadcast to %s' % group_to_send)
                logger.info("Sent jabber broadcast on behalf of user %s" % request.user)
            except PingBotException as e:
                messages.error(request, e)

    else:
        form = JabberBroadcastForm()
        form.fields['group'].choices = allchoices
        logger.debug("Generated broadcast form for user %s containing %s groups" % (
            request.user, len(form.fields['group'].choices)))

    context = {'form': form}
    return render(request, 'services/openfire/broadcast.html', context=context)


@login_required
@permission_required(ACCESS_PERM)
def set_jabber_password(request):
    logger.debug("set_jabber_password called by user %s" % request.user)
    if request.method == 'POST':
        logger.debug("Received POST request with form.")
        form = ServicePasswordForm(request.POST)
        logger.debug("Form is valid: %s" % form.is_valid())
        if form.is_valid() and OpenfireTasks.has_account(request.user):
            password = form.cleaned_data['password']
            logger.debug("Form contains password of length %s" % len(password))
            result = OpenfireManager.update_user_pass(request.user.openfire.username, password=password)
            if result != "":
                logger.info("Successfully set jabber password for user %s" % request.user)
                messages.success(request, 'Set jabber password.')
            else:
                logger.error("Failed to install custom jabber password for user %s" % request.user)
                messages.error(request, 'An error occurred while processing your jabber account.')
            return redirect("services:services")
    else:
        logger.debug("Request is not type POST - providing empty form.")
        form = ServicePasswordForm()

    logger.debug("Rendering form for user %s" % request.user)
    context = {'form': form, 'service': 'Jabber'}
    return render(request, 'services/service_password.html', context=context)



from django.conf.urls import url

from . import views

app_name = 'corputils'
urlpatterns = [
    url(r'^$', views.corpstats_view, name='view'),
    url(r'^add/$', views.corpstats_add, name='add'),
    url(r'^(?P<corp_id>(\d)*)/$', views.corpstats_view, name='view_corp'),
    url(r'^(?P<corp_id>(\d)+)/update/$', views.corpstats_update, name='update'),
    url(r'^search/$', views.corpstats_search, name='search'),
    ]



import logging

from django.conf import settings
from django.template.loader import render_to_string
from allianceauth.notifications import notify

from allianceauth import hooks
from allianceauth.services.hooks import ServicesHook
from .tasks import MumbleTasks
from .models import MumbleUser
from .urls import urlpatterns

logger = logging.getLogger(__name__)


class MumbleService(ServicesHook):
    def __init__(self):
        ServicesHook.__init__(self)
        self.name = 'mumble'
        self.urlpatterns = urlpatterns
        self.service_url = settings.MUMBLE_URL
        self.access_perm = 'mumble.access_mumble'
        self.service_ctrl_template = 'services/mumble/mumble_service_ctrl.html'
        self.name_format = '[{corp_ticker}]{character_name}'

    def delete_user(self, user, notify_user=False):
        logging.debug("Deleting user %s %s account" % (user, self.name))
        try:
            if user.mumble.delete():
                if notify_user:
                    notify(user, 'Mumble Account Disabled', level='danger')
                return True
            return False
        except MumbleUser.DoesNotExist:
            logging.debug("User does not have a mumble account")

    def update_groups(self, user):
        logger.debug("Updating %s groups for %s" % (self.name, user))
        if MumbleTasks.has_account(user):
            MumbleTasks.update_groups.delay(user.pk)

    def validate_user(self, user):
        if MumbleTasks.has_account(user) and not self.service_active_for_user(user):
            self.delete_user(user, notify_user=True)

    def update_all_groups(self):
        logger.debug("Updating all %s groups" % self.name)
        MumbleTasks.update_all_groups.delay()

    def service_active_for_user(self, user):
        return user.has_perm(self.access_perm)

    def render_services_ctrl(self, request):
        urls = self.Urls()
        urls.auth_activate = 'mumble:activate'
        urls.auth_deactivate = 'mumble:deactivate'
        urls.auth_reset_password = 'mumble:reset_password'
        urls.auth_set_password = 'mumble:set_password'

        return render_to_string(self.service_ctrl_template, {
            'service_name': self.title,
            'urls': urls,
            'service_url': self.service_url,
            'connect_url': request.user.mumble.username + '@' + self.service_url if MumbleTasks.has_account(request.user) else self.service_url,
            'username': request.user.mumble.username if MumbleTasks.has_account(request.user) else '',
        }, request=request)


@hooks.register('services_hook')
def register_mumble_service():
    return MumbleService()



from django.contrib.auth.backends import ModelBackend
from django.contrib.auth.models import Permission
from django.contrib.auth.models import User
import logging
from .models import UserProfile, CharacterOwnership, OwnershipRecord


logger = logging.getLogger(__name__)


class StateBackend(ModelBackend):
    @staticmethod
    def _get_state_permissions(user_obj):
        profile_state_field = UserProfile._meta.get_field('state')
        user_state_query = 'state__%s__user' % profile_state_field.related_query_name()
        return Permission.objects.filter(**{user_state_query: user_obj})

    def get_state_permissions(self, user_obj, obj=None):
        return self._get_permissions(user_obj, obj, 'state')

    def get_all_permissions(self, user_obj, obj=None):
        if not user_obj.is_active or user_obj.is_anonymous or obj is not None:
            return set()
        if not hasattr(user_obj, '_perm_cache'):
            user_obj._perm_cache = self.get_user_permissions(user_obj)
            user_obj._perm_cache.update(self.get_group_permissions(user_obj))
            user_obj._perm_cache.update(self.get_state_permissions(user_obj))
        return user_obj._perm_cache

    def authenticate(self, token=None):
        if not token:
            return None
        try:
            ownership = CharacterOwnership.objects.get(character__character_id=token.character_id)
            if ownership.owner_hash == token.character_owner_hash:
                logger.debug('Authenticating {0} by ownership of character {1}'.format(ownership.user, token.character_name))
                return ownership.user
            else:
                logger.debug('{0} has changed ownership. Creating new user account.'.format(token.character_name))
                ownership.delete()
                return self.create_user(token)
        except CharacterOwnership.DoesNotExist:
            try:
                # insecure legacy main check for pre-sso registration auth installs
                profile = UserProfile.objects.get(main_character__character_id=token.character_id)
                logger.debug('Authenticating {0} by their main character {1} without active ownership.'.format(profile.user, profile.main_character))
                # attach an ownership
                token.user = profile.user
                CharacterOwnership.objects.create_by_token(token)
                return profile.user
            except UserProfile.DoesNotExist:
                # now we check historical records to see if this is a returning user
                records = OwnershipRecord.objects.filter(owner_hash=token.character_owner_hash).filter(character__character_id=token.character_id)
                if records.exists():
                    # we've seen this character owner before. Re-attach to their old user account
                    user = records[0].user
                    token.user = user
                    co = CharacterOwnership.objects.create_by_token(token)
                    logger.debug('Authenticating {0} by matching owner hash record of character {1}'.format(user, co.character))
                    if not user.profile.main_character:
                        # set this as their main by default if they have none
                        user.profile.main_character = co.character
                        user.profile.save()
                    return user
            logger.debug('Unable to authenticate character {0}. Creating new user.'.format(token.character_name))
            return self.create_user(token)

    def create_user(self, token):
        username = self.iterate_username(token.character_name)  # build unique username off character name
        user = User.objects.create_user(username, is_active=False)  # prevent login until email set
        user.set_unusable_password()  # prevent login via password
        user.save()
        token.user = user
        co = CharacterOwnership.objects.create_by_token(token)  # assign ownership to this user
        user.profile.main_character = co.character  # assign main character as token character
        user.profile.save()
        logger.debug('Created new user {0}'.format(user))
        return user

    @staticmethod
    def iterate_username(name):
        name = str.replace(name, "'", "")
        name = str.replace(name, ' ', '_')
        if User.objects.filter(username__startswith=name).exists():
            u = User.objects.filter(username__startswith=name)
            num = len(u)
            username = "%s_%s" % (name, num)
            while u.filter(username=username).exists():
                num += 1
                username = "%s_%s" % (name, num)
        else:
            username = name
        return username



from django.conf.urls import url, include

from . import views

app_name = 'xenforo'

module_urls = [
    # XenForo service control
    url(r'^activate/$', views.activate_xenforo_forum, name='activate'),
    url(r'^deactivate/$', views.deactivate_xenforo_forum, name='deactivate'),
    url(r'^reset_password/$', views.reset_xenforo_password, name='reset_password'),
    url(r'^set_password/$', views.set_xenforo_password, name='set_password'),
]

urlpatterns = [
    url(r'^xenforo/', include((module_urls, app_name), namespace=app_name)),
]



from django.contrib import admin
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin
from django.contrib.auth.models import User as BaseUser, Permission as BasePermission
from django.utils.text import slugify
from django.db.models import Q
from allianceauth.services.hooks import ServicesHook
from django.db.models.signals import pre_save, post_save, pre_delete, post_delete, m2m_changed
from django.dispatch import receiver
from allianceauth.authentication.models import State, get_guest_state, CharacterOwnership, UserProfile, OwnershipRecord
from allianceauth.hooks import get_hooks
from allianceauth.eveonline.models import EveCharacter
from django.forms import ModelForm


def make_service_hooks_update_groups_action(service):
    """
    Make a admin action for the given service
    :param service: services.hooks.ServicesHook
    :return: fn to update services groups for the selected users
    """
    def update_service_groups(modeladmin, request, queryset):
        for user in queryset:  # queryset filtering doesn't work here?
            service.update_groups(user)

    update_service_groups.__name__ = str('update_{}_groups'.format(slugify(service.name)))
    update_service_groups.short_description = "Sync groups for selected {} accounts".format(service.title)
    return update_service_groups


def make_service_hooks_sync_nickname_action(service):
    """
    Make a sync_nickname admin action for the given service
    :param service: services.hooks.ServicesHook
    :return: fn to sync nickname for the selected users
    """
    def sync_nickname(modeladmin, request, queryset):
        for user in queryset:  # queryset filtering doesn't work here?
            service.sync_nickname(user)

    sync_nickname.__name__ = str('sync_{}_nickname'.format(slugify(service.name)))
    sync_nickname.short_description = "Sync nicknames for selected {} accounts".format(service.title)
    return sync_nickname


class QuerysetModelForm(ModelForm):
    # allows specifying FK querysets through kwarg
    def __init__(self, querysets=None, *args, **kwargs):
        querysets = querysets or {}
        super().__init__(*args, **kwargs)
        for field, qs in querysets.items():
            self.fields[field].queryset = qs


class UserProfileInline(admin.StackedInline):
    model = UserProfile
    readonly_fields = ('state',)
    form = QuerysetModelForm
    verbose_name = ''
    verbose_name_plural = 'Profile'

    def get_formset(self, request, obj=None, **kwargs):
        # main_character field can only show current value or unclaimed alts
        # if superuser, allow selecting from any unclaimed main
        query = Q()
        if obj and obj.profile.main_character:
            query |= Q(pk=obj.profile.main_character_id)
            if request.user.is_superuser:
                query |= Q(userprofile__isnull=True)
            else:
                query |= Q(character_ownership__user=obj)
        qs = EveCharacter.objects.filter(query)
        formset = super().get_formset(request, obj=obj, **kwargs)

        def get_kwargs(self, index):
            return {'querysets': {'main_character': EveCharacter.objects.filter(query)}}
        formset.get_form_kwargs = get_kwargs
        return formset

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


class UserAdmin(BaseUserAdmin):
    """
    Extending Django's UserAdmin model
    """
    def get_actions(self, request):
        actions = super(BaseUserAdmin, self).get_actions(request)

        for hook in get_hooks('services_hook'):
            svc = hook()
            # Check update_groups is redefined/overloaded
            if svc.update_groups.__module__ != ServicesHook.update_groups.__module__:
                action = make_service_hooks_update_groups_action(svc)
                actions[action.__name__] = (action,
                                            action.__name__,
                                            action.short_description)
            # Create sync nickname action if service implements it
            if svc.sync_nickname.__module__ != ServicesHook.sync_nickname.__module__:
                action = make_service_hooks_sync_nickname_action(svc)
                actions[action.__name__] = (action,
                                            action.__name__,
                                            action.short_description)

        return actions
    list_filter = BaseUserAdmin.list_filter + ('profile__state',)
    inlines = BaseUserAdmin.inlines + [UserProfileInline]
    list_display = ('username', 'email', 'get_main_character', 'get_state', 'is_active')

    def get_main_character(self, obj):
        return obj.profile.main_character
    get_main_character.short_description = "Main Character"

    def get_state(self, obj):
        return obj.profile.state
    get_state.short_description = "State"

    def has_change_permission(self, request, obj=None):
        return request.user.has_perm('auth.change_user')

    def has_add_permission(self, request, obj=None):
        return request.user.has_perm('auth.add_user')

    def has_delete_permission(self, request, obj=None):
        return request.user.has_perm('auth.delete_user')


@admin.register(State)
class StateAdmin(admin.ModelAdmin):
    fieldsets = (
        (None, {
            'fields': ('name', 'permissions', 'priority'),
        }),
        ('Membership', {
            'fields': ('public', 'member_characters', 'member_corporations', 'member_alliances'),
        })
    )
    filter_horizontal = ['member_characters', 'member_corporations', 'member_alliances', 'permissions']
    list_display = ('name', 'priority', 'user_count')

    def has_delete_permission(self, request, obj=None):
        if obj == get_guest_state():
            return False
        return super(StateAdmin, self).has_delete_permission(request, obj=obj)

    def get_fieldsets(self, request, obj=None):
        if obj == get_guest_state():
            return (
                (None, {
                    'fields': ('permissions', 'priority'),
                }),
            )
        return super(StateAdmin, self).get_fieldsets(request, obj=obj)

    @staticmethod
    def user_count(obj):
        return obj.userprofile_set.all().count()


class BaseOwnershipAdmin(admin.ModelAdmin):
    list_display = ('user', 'character')
    search_fields = ('user__username', 'character__character_name', 'character__corporation_name', 'character__alliance_name')

    def get_readonly_fields(self, request, obj=None):
        if obj and obj.pk:
            return 'owner_hash', 'character'
        return tuple()


@admin.register(OwnershipRecord)
class OwnershipRecordAdmin(BaseOwnershipAdmin):
    list_display = BaseOwnershipAdmin.list_display + ('created',)


@admin.register(CharacterOwnership)
class CharacterOwnershipAdmin(BaseOwnershipAdmin):
    def has_add_permission(self, request):
        return False


class PermissionAdmin(admin.ModelAdmin):
    actions = None
    readonly_fields = [field.name for field in BasePermission._meta.fields]
    list_display = ('admin_name', 'name', 'codename', 'content_type')
    list_filter = ('content_type__app_label',)

    @staticmethod
    def admin_name(obj):
        return str(obj)

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def has_module_permission(self, request):
        return True

    def has_change_permission(self, request, obj=None):
        # can see list but not edit it
        return not obj


# Hack to allow registration of django.contrib.auth models in our authentication app
class User(BaseUser):
    class Meta:
        proxy = True
        verbose_name = BaseUser._meta.verbose_name
        verbose_name_plural = BaseUser._meta.verbose_name_plural


class Permission(BasePermission):
    class Meta:
        proxy = True
        verbose_name = BasePermission._meta.verbose_name
        verbose_name_plural = BasePermission._meta.verbose_name_plural


try:
    admin.site.unregister(BaseUser)
finally:
    admin.site.register(User, UserAdmin)
    admin.site.register(Permission, PermissionAdmin)


@receiver(pre_save, sender=User)
def redirect_pre_save(sender, signal=None, *args, **kwargs):
    pre_save.send(BaseUser, *args, **kwargs)


@receiver(post_save, sender=User)
def redirect_post_save(sender, signal=None, *args, **kwargs):
    post_save.send(BaseUser, *args, **kwargs)


@receiver(pre_delete, sender=User)
def redirect_pre_delete(sender, signal=None, *args, **kwargs):
    pre_delete.send(BaseUser, *args, **kwargs)


@receiver(post_delete, sender=User)
def redirect_post_delete(sender, signal=None, *args, **kwargs):
    post_delete.send(BaseUser, *args, **kwargs)


@receiver(m2m_changed, sender=User.groups.through)
def redirect_m2m_changed_groups(sender, signal=None, *args, **kwargs):
    m2m_changed.send(BaseUser, *args, **kwargs)


@receiver(m2m_changed, sender=User.user_permissions.through)
def redirect_m2m_changed_permissions(sender, signal=None, *args, **kwargs):
    m2m_changed.send(BaseUser, *args, **kwargs)



from django.conf.urls import url

from . import views

app_name = 'fleetup'

urlpatterns = [
    url(r'^$', views.fleetup_view, name='view'),
    url(r'^fittings/$', views.fleetup_fittings, name='fittings'),
    url(r'^fittings/(?P<fittingnumber>[0-9]+)/$', views.fleetup_fitting, name='fitting'),
    url(r'^doctrines/$', views.fleetup_doctrines, name='doctrines'),
    url(r'^characters/$', views.fleetup_characters, name='characters'),
    url(r'^doctrines/(?P<doctrinenumber>[0-9]+)/$', views.fleetup_doctrine, name='doctrine'),
]



import logging

from django.conf import settings
from django.template.loader import render_to_string

from allianceauth import hooks
from allianceauth.services.hooks import ServicesHook, MenuItemHook
from .tasks import OpenfireTasks
from .urls import urlpatterns

logger = logging.getLogger(__name__)


class OpenfireService(ServicesHook):
    def __init__(self):
        ServicesHook.__init__(self)
        self.name = 'openfire'
        self.urlpatterns = urlpatterns
        self.service_url = settings.JABBER_URL
        self.access_perm = 'openfire.access_openfire'
        self.name_format = '{character_name}'

    @property
    def title(self):
        return "Jabber"

    def delete_user(self, user, notify_user=False):
        logger.debug('Deleting user %s %s account' % (user, self.name))
        return OpenfireTasks.delete_user(user, notify_user=notify_user)

    def validate_user(self, user):
        logger.debug('Validating user %s %s account' % (user, self.name))
        if OpenfireTasks.has_account(user) and not self.service_active_for_user(user):
            self.delete_user(user, notify_user=True)

    def update_groups(self, user):
        logger.debug('Updating %s groups for %s' % (self.name, user))
        if OpenfireTasks.has_account(user):
            OpenfireTasks.update_groups.delay(user.pk)

    def update_all_groups(self):
        logger.debug('Update all %s groups called' % self.name)
        OpenfireTasks.update_all_groups.delay()

    def service_active_for_user(self, user):
        return user.has_perm(self.access_perm)

    def render_services_ctrl(self, request):
        """
        Example for rendering the service control panel row
        You can override the default template and create a
        custom one if you wish.
        :param request:
        :return:
        """
        urls = self.Urls()
        urls.auth_activate = 'openfire:activate'
        urls.auth_deactivate = 'openfire:deactivate'
        urls.auth_set_password = 'openfire:set_password'
        urls.auth_reset_password = 'openfire:reset_password'
        return render_to_string(self.service_ctrl_template, {
            'service_name': self.title,
            'urls': urls,
            'service_url': self.service_url,
            'username': request.user.openfire.username if OpenfireTasks.has_account(request.user) else ''
        }, request=request)


@hooks.register('services_hook')
def register_service():
    return OpenfireService()


class JabberBroadcast(MenuItemHook):
    def __init__(self):
        MenuItemHook.__init__(self,
                              'Jabber Broadcast',
                              'fa fa-lock fa-fw fa-bullhorn',
                              'openfire:broadcast')

    def render(self, request):
        if request.user.has_perm('auth.jabber_broadcast') or request.user.has_perm('auth.jabber_broadcast_all'):
            return MenuItemHook.render(self, request)
        return ''


class FleetBroadcastFormatter(MenuItemHook):
    def __init__(self):
        MenuItemHook.__init__(self,
                              'Fleet Broadcast Formatter',
                              'fa fa-lock fa-fw fa-space-shuttle',
                              'services:fleet_format_tool')

    def render(self, request):
        if request.user.has_perm('auth.jabber_broadcast') or request.user.has_perm('auth.jabber_broadcast_all'):
            return MenuItemHook.render(self, request)
        return ''


@hooks.register('menu_item_hook')
def register_formatter():
    return FleetBroadcastFormatter()


@hooks.register('menu_item_hook')
def register_menu():
    return JabberBroadcast()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-02-23 06:13
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('authentication', '0015_user_profiles'),
        ('teamspeak3', '0004_service_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='StateGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('state', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='authentication.State')),
                ('ts_group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='teamspeak3.TSgroup')),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-08-23 19:46
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('hrapplications', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ApplicationChoice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('choice_text', models.CharField(max_length=200, verbose_name='Choice')),
            ],
        ),
        migrations.AlterField(
            model_name='applicationquestion',
            name='title',
            field=models.CharField(max_length=254, verbose_name='Question'),
        ),
        migrations.AddField(
            model_name='applicationchoice',
            name='question',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='choices', to='hrapplications.ApplicationQuestion'),
        ),
    ]



from django.conf.urls import include, url
from allianceauth.hooks import get_hooks

from . import views

urlpatterns = [
    # Services
    url(r'^services/', include(([
        url(r'^$', views.services_view, name='services'),
        # Tools
        url(r'^tool/fleet_formatter_tool/$', views.fleet_formatter_view, name='fleet_format_tool'),
    ], 'services'), namespace='services')),
]

# Append hooked service urls
services = get_hooks('services_hook')
for svc in services:
    urlpatterns += svc().urlpatterns



from django.contrib import admin
from .models import DiscourseUser


class DiscourseUserAdmin(admin.ModelAdmin):
        list_display = ('user',)
        search_fields = ('user__username',)

admin.site.register(DiscourseUser, DiscourseUserAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-05 21:39
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('eveonline', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='GroupDescription',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(max_length=512)),
                ('group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='auth.Group', unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='GroupRequest',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.CharField(max_length=254)),
                ('leave_request', models.BooleanField(default=0)),
                ('group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='auth.Group')),
                ('main_char', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='eveonline.EveCharacter')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='HiddenGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='auth.Group', unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='OpenGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('group', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='auth.Group')),
            ],
        ),
    ]



from django.template.loader import render_to_string

from allianceauth import hooks
from allianceauth.services.hooks import ServicesHook
from .urls import urlpatterns


class ExampleService(ServicesHook):
    def __init__(self):
        ServicesHook.__init__(self)
        self.urlpatterns = urlpatterns
        self.service_url = 'http://exampleservice.example.com'
        self.name = 'example'

    """
    Overload base methods here to implement functionality
    """

    def render_services_ctrl(self, request):
        """
        Example for rendering the service control panel row
        You can override the default template and create a
        custom one if you wish.
        :param request:
        :return:
        """
        urls = self.Urls()
        urls.auth_activate = 'auth_example_activate'
        urls.auth_deactivate = 'auth_example_deactivate'
        urls.auth_reset_password = 'auth_example_reset_password'
        urls.auth_set_password = 'auth_example_set_password'
        return render_to_string(self.service_ctrl_template, {
            'service_name': self.title,
            'urls': urls,
            'service_url': self.service_url,
            'username': 'example username'
        }, request=request)


@hooks.register('services_hook')
def register_service():
    return ExampleService()



from django.contrib import admin
from django.contrib.auth.models import Group as BaseGroup
from django.db.models.signals import pre_save, post_save, pre_delete, post_delete, m2m_changed
from django.dispatch import receiver
from .models import AuthGroup
from .models import GroupRequest


class AuthGroupInlineAdmin(admin.StackedInline):
    model = AuthGroup
    filter_horizontal = ('group_leaders',)
    fields = ('description', 'group_leaders', 'internal', 'hidden', 'open', 'public')
    verbose_name_plural = 'Auth Settings'
    verbose_name = ''

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        return request.user.has_perm('auth.change_group')


class GroupAdmin(admin.ModelAdmin):
    filter_horizontal = ('permissions',)
    inlines = (AuthGroupInlineAdmin,)


class Group(BaseGroup):
    class Meta:
        proxy = True
        verbose_name = BaseGroup._meta.verbose_name
        verbose_name_plural = BaseGroup._meta.verbose_name_plural

try:
    admin.site.unregister(BaseGroup)
finally:
    admin.site.register(Group, GroupAdmin)


admin.site.register(GroupRequest)


@receiver(pre_save, sender=Group)
def redirect_pre_save(sender, signal=None, *args, **kwargs):
    pre_save.send(BaseGroup, *args, **kwargs)


@receiver(post_save, sender=Group)
def redirect_post_save(sender, signal=None, *args, **kwargs):
    post_save.send(BaseGroup, *args, **kwargs)


@receiver(pre_delete, sender=Group)
def redirect_pre_delete(sender, signal=None, *args, **kwargs):
    pre_delete.send(BaseGroup, *args, **kwargs)


@receiver(post_delete, sender=Group)
def redirect_post_delete(sender, signal=None, *args, **kwargs):
    post_delete.send(BaseGroup, *args, **kwargs)


@receiver(m2m_changed, sender=Group.permissions.through)
def redirect_m2m_changed_permissions(sender, signal=None, *args, **kwargs):
    m2m_changed.send(BaseGroup, *args, **kwargs)


from django.conf import settings
from django.template.loader import render_to_string

from allianceauth import hooks
from allianceauth.services.hooks import ServicesHook
from .tasks import Ips4Tasks
from .urls import urlpatterns


class Ips4Service(ServicesHook):
    def __init__(self):
        ServicesHook.__init__(self)
        self.name = 'ips4'
        self.urlpatterns = urlpatterns
        self.service_url = settings.IPS4_URL
        self.access_perm = 'ips4.access_ips4'
        self.name_format = '{character_name}'

    @property
    def title(self):
        return 'IPS4'

    def service_active_for_user(self, user):
        return user.has_perm(self.access_perm)

    def render_services_ctrl(self, request):
        """
        Example for rendering the service control panel row
        You can override the default template and create a
        custom one if you wish.
        :param request:
        :return:
        """
        urls = self.Urls()
        urls.auth_activate = 'ips4:activate'
        urls.auth_deactivate = 'ips4:deactivate'
        urls.auth_reset_password = 'ips4:reset_password'
        urls.auth_set_password = 'ips4:set_password'
        return render_to_string(self.service_ctrl_template, {
            'service_name': self.title,
            'urls': urls,
            'service_url': self.service_url,
            'username': request.user.ips4.username if Ips4Tasks.has_account(request.user) else ''
        }, request=request)


@hooks.register('services_hook')
def register_service():
    return Ips4Service()



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-01-18 20:21
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('srp', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='srpuserrequest',
            name='srp_status',
            field=models.CharField(choices=[('Pending', 'Pending'), ('Approved', 'Approved'), ('Rejected', 'Rejected')], default='Pending', max_length=8),
        ),
    ]



import random
import string
import requests
import json

from django.conf import settings

import logging

logger = logging.getLogger(__name__)


class XenForoManager:
    def __init__(self):
        if not settings.XENFORO_ENDPOINT:
            logger.debug("Could not find XenForo endpoint")
        if not settings.XENFORO_APIKEY:
            logger.debug("XenForo API Key not found")
        pass

    @staticmethod
    def __sanitize_username(username):
        sanitized = username.replace(" ", "_")
        return sanitized

    @staticmethod
    def __generate_password():
        return ''.join([random.choice(string.ascii_letters + string.digits) for n in range(16)])

    @staticmethod
    def exec_http_request(http_params):
        default_params = {
            'hash': settings.XENFORO_APIKEY
        }
        http_params.update(default_params)
        r = requests.get(settings.XENFORO_ENDPOINT, params=http_params)
        return r

    @staticmethod
    def add_user(username, email):

        sanitized = XenForoManager.__sanitize_username(username)
        password = XenForoManager.__generate_password()

        data = {
            'action': 'register',
            'username': sanitized,
            'password': password,
            'email': email,
            'group': settings.XENFORO_DEFAULT_GROUP,
            'visible': '1',
            'user_state': 'valid'
        }

        r = XenForoManager.exec_http_request(data)

        # check if the user already exist but was disabled
        if r.status_code != 200:
            if json.loads(r.text)['error'] == 7:
                data = XenForoManager.reactivate_user(sanitized)
                return data

        response = {
            'response': {
                'message': r.text,
                'status_code': r.status_code
            }
        }
        data.update(response)
        return data

    @staticmethod
    def reset_password(username):

        password = XenForoManager.__generate_password()

        data = {
            'action': 'editUser',
            'user': username,
            'password': password
        }

        r = XenForoManager.exec_http_request(data)

        response = {
            'response': {
                'message': r.text,
                'status_code': r.status_code
            }
        }
        data.update(response)
        return data

    @staticmethod
    def disable_user(username):
        data = {
            'action': 'editUser',
            'user': username,
            'remove_groups': settings.XENFORO_DEFAULT_GROUP
        }
        r = XenForoManager.exec_http_request(data)
        return r

    @staticmethod
    def reactivate_user(username):
        data = {
            'action': 'editUser',
            'user': username,
            'add_groups': settings.XENFORO_DEFAULT_GROUP,
            'password': XenForoManager.__generate_password()
        }
        r = XenForoManager.exec_http_request(data)
        response = {
            'response': {
                'message': r.text,
                'status_code': r.status_code
            },
            'username': username
        }
        data.update(response)
        return data

    @staticmethod
    def update_user_password(username, raw_password):
        data = {
            'action': 'editUser',
            'user': username,
            'password': raw_password
        }
        r = XenForoManager.exec_http_request(data)
        response = {
            'response': {
                'message': r.text,
                'status_code': r.status_code
            },
            'username': username
        }
        data.update(response)
        return data



from unittest import mock
from io import StringIO
from django.test import TestCase
from django.contrib.auth.models import User
from allianceauth.tests.auth_utils import AuthUtils
from .models import CharacterOwnership, UserProfile, State, get_guest_state, OwnershipRecord
from .backends import StateBackend
from .tasks import check_character_ownership
from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo, EveAllianceInfo
from esi.models import Token
from esi.errors import IncompleteResponseError
from allianceauth.authentication.decorators import main_character_required
from django.test.client import RequestFactory
from django.http.response import HttpResponse
from django.contrib.auth.models import AnonymousUser
from django.conf import settings
from django.shortcuts import reverse
from django.core.management import call_command
from urllib import parse

MODULE_PATH = 'allianceauth.authentication'


class DecoratorTestCase(TestCase):
    @staticmethod
    @main_character_required
    def dummy_view(*args, **kwargs):
        return HttpResponse(status=200)

    @classmethod
    def setUpTestData(cls):
        cls.main_user = AuthUtils.create_user('main_user', disconnect_signals=True)
        cls.no_main_user = AuthUtils.create_user('no_main_user', disconnect_signals=True)
        main_character = EveCharacter.objects.create(
            character_id=1,
            character_name='Main Character',
            corporation_id=1,
            corporation_name='Corp',
            corporation_ticker='CORP',
        )
        CharacterOwnership.objects.create(user=cls.main_user, character=main_character, owner_hash='1')
        cls.main_user.profile.main_character = main_character

    def setUp(self):
        self.request = RequestFactory().get('/test/')

    @mock.patch(MODULE_PATH + '.decorators.messages')
    def test_login_redirect(self, m):
        setattr(self.request, 'user', AnonymousUser())
        response = self.dummy_view(self.request)
        self.assertEqual(response.status_code, 302)
        url = getattr(response, 'url', None)
        self.assertEqual(parse.urlparse(url).path,  reverse(settings.LOGIN_URL))

    @mock.patch(MODULE_PATH + '.decorators.messages')
    def test_main_character_redirect(self, m):
        setattr(self.request, 'user', self.no_main_user)
        response = self.dummy_view(self.request)
        self.assertEqual(response.status_code, 302)
        url = getattr(response, 'url', None)
        self.assertEqual(url, reverse('authentication:dashboard'))

    @mock.patch(MODULE_PATH + '.decorators.messages')
    def test_successful_request(self, m):
        setattr(self.request, 'user', self.main_user)
        response = self.dummy_view(self.request)
        self.assertEqual(response.status_code, 200)


class BackendTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.main_character = EveCharacter.objects.create(
            character_id=1,
            character_name='Main Character',
            corporation_id=1,
            corporation_name='Corp',
            corporation_ticker='CORP',
        )
        cls.alt_character = EveCharacter.objects.create(
            character_id=2,
            character_name='Alt Character',
            corporation_id=1,
            corporation_name='Corp',
            corporation_ticker='CORP',
        )
        cls.unclaimed_character = EveCharacter.objects.create(
            character_id=3,
            character_name='Unclaimed Character',
            corporation_id=1,
            corporation_name='Corp',
            corporation_ticker='CORP',
        )
        cls.user = AuthUtils.create_user('test_user', disconnect_signals=True)
        cls.old_user = AuthUtils.create_user('old_user', disconnect_signals=True)
        AuthUtils.disconnect_signals()
        CharacterOwnership.objects.create(user=cls.user, character=cls.main_character, owner_hash='1')
        CharacterOwnership.objects.create(user=cls.user, character=cls.alt_character, owner_hash='2')
        UserProfile.objects.update_or_create(user=cls.user, defaults={'main_character': cls.main_character})
        AuthUtils.connect_signals()

    def test_authenticate_main_character(self):
        t = Token(character_id=self.main_character.character_id, character_owner_hash='1')
        user = StateBackend().authenticate(token=t)
        self.assertEquals(user, self.user)

    def test_authenticate_alt_character(self):
        t = Token(character_id=self.alt_character.character_id, character_owner_hash='2')
        user = StateBackend().authenticate(token=t)
        self.assertEquals(user, self.user)

    def test_authenticate_unclaimed_character(self):
        t = Token(character_id=self.unclaimed_character.character_id, character_name=self.unclaimed_character.character_name, character_owner_hash='3')
        user = StateBackend().authenticate(token=t)
        self.assertNotEqual(user, self.user)
        self.assertEqual(user.username, 'Unclaimed_Character')
        self.assertEqual(user.profile.main_character, self.unclaimed_character)

    def test_authenticate_character_record(self):
        t = Token(character_id=self.unclaimed_character.character_id, character_name=self.unclaimed_character.character_name, character_owner_hash='4')
        record = OwnershipRecord.objects.create(user=self.old_user, character=self.unclaimed_character, owner_hash='4')
        user = StateBackend().authenticate(t)
        self.assertEqual(user, self.old_user)
        self.assertTrue(CharacterOwnership.objects.filter(owner_hash='4', user=self.old_user).exists())
        self.assertTrue(user.profile.main_character)

    def test_iterate_username(self):
        t = Token(character_id=self.unclaimed_character.character_id,
                  character_name=self.unclaimed_character.character_name, character_owner_hash='3')
        username = StateBackend().authenticate(token=t).username
        t.character_owner_hash = '4'
        username_1 = StateBackend().authenticate(token=t).username
        t.character_owner_hash = '5'
        username_2 = StateBackend().authenticate(token=t).username
        self.assertNotEqual(username, username_1, username_2)
        self.assertTrue(username_1.endswith('_1'))
        self.assertTrue(username_2.endswith('_2'))


class CharacterOwnershipTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.user = AuthUtils.create_user('user', disconnect_signals=True)
        cls.alt_user = AuthUtils.create_user('alt_user', disconnect_signals=True)
        cls.character = EveCharacter.objects.create(
            character_id=1,
            character_name='Main Character',
            corporation_id=1,
            corporation_name='Corp',
            corporation_ticker='CORP',
        )

    def test_create_ownership(self):
        Token.objects.create(
            user=self.user,
            character_id=self.character.character_id,
            character_name=self.character.character_name,
            character_owner_hash='1',
        )
        co = CharacterOwnership.objects.get(character=self.character)
        self.assertEquals(co.user, self.user)
        self.assertEquals(co.owner_hash, '1')

    def test_transfer_ownership(self):
        Token.objects.create(
            user=self.user,
            character_id=self.character.character_id,
            character_name=self.character.character_name,
            character_owner_hash='1',
        )
        Token.objects.create(
            user=self.alt_user,
            character_id=self.character.character_id,
            character_name=self.character.character_name,
            character_owner_hash='2',
        )
        co = CharacterOwnership.objects.get(character=self.character)
        self.assertNotEqual(self.user, co.user)
        self.assertEquals(self.alt_user, co.user)

    def test_clear_main_character(self):
        Token.objects.create(
            user=self.user,
            character_id=self.character.character_id,
            character_name=self.character.character_name,
            character_owner_hash='1',
        )
        self.user.profile.main_character = self.character
        self.user.profile.save()
        Token.objects.create(
            user=self.alt_user,
            character_id=self.character.character_id,
            character_name=self.character.character_name,
            character_owner_hash='2',
        )
        self.user = User.objects.get(pk=self.user.pk)
        self.assertIsNone(self.user.profile.main_character)


class StateTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.user = AuthUtils.create_user('test_user', disconnect_signals=True)
        AuthUtils.add_main_character(cls.user, 'Test Character', '1', corp_id='1', alliance_id='1',
                                     corp_name='Test Corp', alliance_name='Test Alliance')
        cls.guest_state = get_guest_state()
        cls.test_character = EveCharacter.objects.get(character_id='1')
        cls.test_corporation = EveCorporationInfo.objects.create(corporation_id='1', corporation_name='Test Corp',
                                                                  corporation_ticker='TEST', member_count=1)
        cls.test_alliance = EveAllianceInfo.objects.create(alliance_id='1', alliance_name='Test Alliance',
                                                            alliance_ticker='TEST', executor_corp_id='1')
        cls.member_state = State.objects.create(
            name='Test Member',
            priority=150,
        )

    def _refresh_user(self):
        self.user = User.objects.get(pk=self.user.pk)

    def test_state_assignment_on_character_change(self):
        self.member_state.member_characters.add(self.test_character)
        self._refresh_user()
        self.assertEquals(self.user.profile.state, self.member_state)

        self.member_state.member_characters.remove(self.test_character)
        self._refresh_user()
        self.assertEquals(self.user.profile.state, self.guest_state)

    def test_state_assignment_on_corporation_change(self):
        self.member_state.member_corporations.add(self.test_corporation)
        self._refresh_user()
        self.assertEquals(self.user.profile.state, self.member_state)

        self.member_state.member_corporations.remove(self.test_corporation)
        self._refresh_user()
        self.assertEquals(self.user.profile.state, self.guest_state)

    def test_state_assignment_on_alliance_addition(self):
        self.member_state.member_alliances.add(self.test_alliance)
        self._refresh_user()
        self.assertEquals(self.user.profile.state, self.member_state)

        self.member_state.member_alliances.remove(self.test_alliance)
        self._refresh_user()
        self.assertEquals(self.user.profile.state, self.guest_state)

    def test_state_assignment_on_higher_priority_state_creation(self):
        self.member_state.member_characters.add(self.test_character)
        higher_state = State.objects.create(
            name='Higher State',
            priority=200,
        )
        higher_state.member_characters.add(self.test_character)
        self._refresh_user()
        self.assertEquals(higher_state, self.user.profile.state)
        higher_state.member_characters.clear()
        self._refresh_user()
        self.assertEquals(self.member_state, self.user.profile.state)
        self.member_state.member_characters.clear()

    def test_state_assignment_on_lower_priority_state_creation(self):
        self.member_state.member_characters.add(self.test_character)
        lower_state = State.objects.create(
            name='Lower State',
            priority=125,
        )
        lower_state.member_characters.add(self.test_character)
        self._refresh_user()
        self.assertEquals(self.member_state, self.user.profile.state)
        lower_state.member_characters.clear()
        self._refresh_user()
        self.assertEquals(self.member_state, self.user.profile.state)
        self.member_state.member_characters.clear()

    def test_state_assignment_on_priority_change(self):
        self.member_state.member_characters.add(self.test_character)
        lower_state = State.objects.create(
            name='Lower State',
            priority=125,
        )
        lower_state.member_characters.add(self.test_character)
        self._refresh_user()
        lower_state.priority = 500
        lower_state.save()
        self._refresh_user()
        self.assertEquals(lower_state, self.user.profile.state)
        lower_state.priority = 125
        lower_state.save()
        self._refresh_user()
        self.assertEquals(self.member_state, self.user.profile.state)

    def test_state_assignment_on_state_deletion(self):
        self.member_state.member_characters.add(self.test_character)
        higher_state = State.objects.create(
            name='Higher State',
            priority=200,
        )
        higher_state.member_characters.add(self.test_character)
        self._refresh_user()
        self.assertEquals(higher_state, self.user.profile.state)
        higher_state.delete()
        self.assertFalse(State.objects.filter(name='Higher State').count())
        self._refresh_user()
        self.assertEquals(self.member_state, self.user.profile.state)

    def test_state_assignment_on_public_toggle(self):
        self.member_state.member_characters.add(self.test_character)
        higher_state = State.objects.create(
            name='Higher State',
            priority=200,
        )
        self._refresh_user()
        self.assertEquals(self.member_state, self.user.profile.state)
        higher_state.public = True
        higher_state.save()
        self._refresh_user()
        self.assertEquals(higher_state, self.user.profile.state)
        higher_state.public = False
        higher_state.save()
        self._refresh_user()
        self.assertEquals(self.member_state, self.user.profile.state)

    def test_state_assignment_on_active_changed(self):
        self.member_state.member_characters.add(self.test_character)
        self.user.is_active = False
        self.user.save()
        self._refresh_user()
        self.assertEquals(self.user.profile.state, self.guest_state)
        self.user.is_active = True
        self.user.save()
        self._refresh_user()
        self.assertEquals(self.user.profile.state, self.member_state)


class CharacterOwnershipCheckTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.user = AuthUtils.create_user('test_user', disconnect_signals=True)
        AuthUtils.add_main_character(cls.user, 'Test Character', '1', corp_id='1', alliance_id='1',
                                     corp_name='Test Corp', alliance_name='Test Alliance')
        cls.character = EveCharacter.objects.get(character_id='1')
        cls.token = Token.objects.create(
            user=cls.user,
            character_id='1',
            character_name='Test',
            character_owner_hash='1',
        )
        cls.ownership = CharacterOwnership.objects.get(character=cls.character)

    @mock.patch(MODULE_PATH + '.tasks.Token.update_token_data')
    def test_no_change_owner_hash(self, update_token_data):
        # makes sure the ownership isn't delete if owner hash hasn't changed
        check_character_ownership(self.ownership)
        self.assertTrue(CharacterOwnership.objects.filter(user=self.user).filter(character=self.character).exists())

    @mock.patch(MODULE_PATH + '.tasks.Token.update_token_data')
    def test_unable_to_update_token_data(self, update_token_data):
        # makes sure ownerships and tokens aren't hellpurged when there's problems with the SSO servers
        update_token_data.side_effect = IncompleteResponseError()
        check_character_ownership(self.ownership)
        self.assertTrue(CharacterOwnership.objects.filter(user=self.user).filter(character=self.character).exists())

        update_token_data.side_effect = KeyError()
        check_character_ownership(self.ownership)
        self.assertTrue(CharacterOwnership.objects.filter(user=self.user).filter(character=self.character).exists())

    @mock.patch(MODULE_PATH + '.tasks.Token.update_token_data')
    @mock.patch(MODULE_PATH + '.tasks.Token.delete')
    @mock.patch(MODULE_PATH + '.tasks.Token.objects.exists')
    @mock.patch(MODULE_PATH + '.tasks.CharacterOwnership.objects.filter')
    def test_owner_hash_changed(self, filter, exists, delete, update_token_data):
        # makes sure the ownership is revoked when owner hash changes
        filter.return_value.exists.return_value = False
        check_character_ownership(self.ownership)
        self.assertTrue(filter.return_value.delete.called)


class ManagementCommandTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.user = AuthUtils.create_user('test user', disconnect_signals=True)
        AuthUtils.add_main_character(cls.user, 'test character', '1', '2', 'test corp', 'test')
        character = UserProfile.objects.get(user=cls.user).main_character
        CharacterOwnership.objects.create(user=cls.user, character=character, owner_hash='test')

    def setUp(self):
        self.stdout = StringIO()

    def test_ownership(self):
        call_command('checkmains', stdout=self.stdout)
        self.assertFalse(UserProfile.objects.filter(main_character__isnull=True).count())
        self.assertNotIn(self.user.username, self.stdout.getvalue())
        self.assertIn('All main characters', self.stdout.getvalue())

    def test_no_ownership(self):
        user = AuthUtils.create_user('v1 user', disconnect_signals=True)
        AuthUtils.add_main_character(user, 'v1 character', '10', '20', 'test corp', 'test')
        self.assertFalse(UserProfile.objects.filter(main_character__isnull=True).count())

        call_command('checkmains', stdout=self.stdout)
        self.assertEqual(UserProfile.objects.filter(main_character__isnull=True).count(), 1)
        self.assertIn(user.username, self.stdout.getvalue())



from django.views.generic.base import View
from django.http import HttpResponseRedirect


class NightModeRedirectView(View):
    SESSION_VAR = 'NIGHT_MODE'

    def get(self, request, *args, **kwargs):
        request.session[self.SESSION_VAR] = not self.night_mode_state(request)
        return HttpResponseRedirect(request.GET.get('next', '/'))

    @classmethod
    def night_mode_state(cls, request):
        try:
            return request.session.get(cls.SESSION_VAR, False)
        except AttributeError:
            # Session is middleware
            # Sometimes request wont have a session attribute
            return False



import logging

from django.contrib.auth.models import User, Permission
from django.db import models, transaction
from django.utils.translation import ugettext_lazy as _
from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo, EveAllianceInfo
from allianceauth.notifications import notify

from .managers import CharacterOwnershipManager, StateManager

logger = logging.getLogger(__name__)


class State(models.Model):
    name = models.CharField(max_length=20, unique=True)
    permissions = models.ManyToManyField(Permission, blank=True)
    priority = models.IntegerField(unique=True,
                                   help_text="Users get assigned the state with the highest priority available to them.")

    member_characters = models.ManyToManyField(EveCharacter, blank=True,
                                               help_text="Characters to which this state is available.")
    member_corporations = models.ManyToManyField(EveCorporationInfo, blank=True,
                                                 help_text="Corporations to whose members this state is available.")
    member_alliances = models.ManyToManyField(EveAllianceInfo, blank=True,
                                              help_text="Alliances to whose members this state is available.")
    public = models.BooleanField(default=False, help_text="Make this state available to any character.")

    objects = StateManager()

    class Meta:
        ordering = ['-priority']

    def __str__(self):
        return self.name

    def available_to_character(self, character):
        return self in State.objects.available_to_character(character)

    def available_to_user(self, user):
        return self in State.objects.available_to_user(user)

    def delete(self, **kwargs):
        with transaction.atomic():
            for profile in self.userprofile_set.all():
                profile.assign_state(state=State.objects.exclude(pk=self.pk).get_for_user(profile.user))
        super(State, self).delete(**kwargs)


def get_guest_state():
    try:
        return State.objects.get(name='Guest')
    except State.DoesNotExist:
        return State.objects.create(name='Guest', priority=0, public=True)


def get_guest_state_pk():
    return get_guest_state().pk


class UserProfile(models.Model):
    class Meta:
        default_permissions = ('change',)

    user = models.OneToOneField(User, related_name='profile', on_delete=models.CASCADE)
    main_character = models.OneToOneField(EveCharacter, blank=True, null=True, on_delete=models.SET_NULL)
    state = models.ForeignKey(State, on_delete=models.SET_DEFAULT, default=get_guest_state_pk)

    def assign_state(self, state=None, commit=True):
        if not state:
            state = State.objects.get_for_user(self.user)
        if self.state != state:
            self.state = state
            if commit:
                logger.info('Updating {} state to {}'.format(self.user, self.state))
                self.save(update_fields=['state'])
                notify(self.user, _('State Changed'),
                       _('Your user state has been changed to %(state)s') % ({'state': state}),
                       'info')
                from allianceauth.authentication.signals import state_changed
                state_changed.send(sender=self.__class__, user=self.user, state=self.state)

    def __str__(self):
        return str(self.user)


class CharacterOwnership(models.Model):
    class Meta:
        default_permissions = ('change', 'delete')
        ordering = ['user', 'character__character_name']

    character = models.OneToOneField(EveCharacter, on_delete=models.CASCADE, related_name='character_ownership')
    owner_hash = models.CharField(max_length=28, unique=True)
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name='character_ownerships')

    objects = CharacterOwnershipManager()

    def __str__(self):
        return "%s: %s" % (self.user, self.character)


class OwnershipRecord(models.Model):
    character = models.ForeignKey(EveCharacter, on_delete=models.CASCADE, related_name='ownership_records')
    owner_hash = models.CharField(max_length=28, db_index=True)
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name='ownership_records')
    created = models.DateTimeField(auto_now=True)

    class Meta:
        ordering = ['-created']

    def __str__(self):
        return "%s: %s on %s" % (self.user, self.character, self.created)


from django.contrib import admin
from .models import Phpbb3User


class Phpbb3UserAdmin(admin.ModelAdmin):
        list_display = ('user', 'username')
        search_fields = ('user__username', 'username')

admin.site.register(Phpbb3User, Phpbb3UserAdmin)



import re
import random
import string
from urllib.parse import urlparse

import sleekxmpp
from django.conf import settings
from ofrestapi.users import Users as ofUsers
from ofrestapi import exception

import logging

logger = logging.getLogger(__name__)


class OpenfireManager:
    def __init__(self):
        pass

    @staticmethod
    def __add_address_to_username(username):
        address = urlparse(settings.OPENFIRE_ADDRESS).netloc.split(":")[0]
        completed_username = username + "@" + address
        return completed_username

    @staticmethod
    def __sanitize_username(username):
        # https://xmpp.org/extensions/xep-0106.html#escaping
        replace = [
            ("\\", "\\5c"),  # Escape backslashes first to double escape existing escape sequences
            ("\"", "\\22"),
            ("&", "\\26"),
            ("'", "\\27"),
            ("/", "\\2f"),
            (":", "\\3a"),
            ("<", "\\3c"),
            (">", "\\3e"),
            ("@", "\\40"),
            ("\u007F", ""),
            ("\uFFFE", ""),
            ("\uFFFF", ""),
            (" ", "\\20"),
        ]

        sanitized = username.strip(' ')

        for find, rep in replace:
            sanitized = sanitized.replace(find, rep)

        return sanitized

    @staticmethod
    def __generate_random_pass():
        return ''.join([random.choice(string.ascii_letters + string.digits) for n in range(16)])

    @staticmethod
    def _sanitize_groupname(name):
        name = name.strip(' _').lower()
        return re.sub('[^\w.-]', '', name)

    @staticmethod
    def add_user(username):
        logger.debug("Adding username %s to openfire." % username)
        try:
            sanitized_username = OpenfireManager.__sanitize_username(username)
            password = OpenfireManager.__generate_random_pass()
            api = ofUsers(settings.OPENFIRE_ADDRESS, settings.OPENFIRE_SECRET_KEY)
            api.add_user(sanitized_username, password)
            logger.info("Added openfire user %s" % username)
        except exception.UserAlreadyExistsException:
            # User exist
            logger.error("Attempting to add a user %s to openfire which already exists on server." % username)
            return "", ""

        return sanitized_username, password

    @staticmethod
    def delete_user(username):
        logger.debug("Deleting user %s from openfire." % username)
        try:
            api = ofUsers(settings.OPENFIRE_ADDRESS, settings.OPENFIRE_SECRET_KEY)
            api.delete_user(username)
            logger.info("Deleted user %s from openfire." % username)
            return True
        except exception.UserNotFoundException:
            logger.error("Attempting to delete a user %s from openfire which was not found on server." % username)
            return False

    @staticmethod
    def lock_user(username):
        logger.debug("Locking openfire user %s" % username)
        api = ofUsers(settings.OPENFIRE_ADDRESS, settings.OPENFIRE_SECRET_KEY)
        api.lock_user(username)
        logger.info("Locked openfire user %s" % username)

    @staticmethod
    def unlock_user(username):
        logger.debug("Unlocking openfire user %s" % username)
        api = ofUsers(settings.OPENFIRE_ADDRESS, settings.OPENFIRE_SECRET_KEY)
        api.unlock_user(username)
        logger.info("Unlocked openfire user %s" % username)

    @staticmethod
    def update_user_pass(username, password=None):
        logger.debug("Updating openfire user %s password." % username)
        try:
            if not password:
                password = OpenfireManager.__generate_random_pass()
            api = ofUsers(settings.OPENFIRE_ADDRESS, settings.OPENFIRE_SECRET_KEY)
            api.update_user(username, password=password)
            logger.info("Updated openfire user %s password." % username)
            return password
        except exception.UserNotFoundException:
            logger.error("Unable to update openfire user %s password - user not found on server." % username)
            return ""

    @classmethod
    def update_user_groups(cls, username, groups):
        logger.debug("Updating openfire user %s groups %s" % (username, groups))
        s_groups = list(map(cls._sanitize_groupname, groups))  # Sanitized group names
        api = ofUsers(settings.OPENFIRE_ADDRESS, settings.OPENFIRE_SECRET_KEY)
        response = api.get_user_groups(username)
        remote_groups = []
        if response:
            remote_groups = response['groupname']
            if isinstance(remote_groups, str):
                remote_groups = [remote_groups]
        remote_groups = list(map(cls._sanitize_groupname, remote_groups))
        logger.debug("Openfire user %s has groups %s" % (username, remote_groups))
        add_groups = []
        del_groups = []
        for g in s_groups:
            if g not in remote_groups:
                add_groups.append(g)
        for g in remote_groups:
            if g not in s_groups:
                del_groups.append(g)
        logger.info(
            "Updating openfire groups for user %s - adding %s, removing %s" % (username, add_groups, del_groups))
        if add_groups:
            api.add_user_groups(username, add_groups)
        if del_groups:
            api.delete_user_groups(username, del_groups)

    @staticmethod
    def delete_user_groups(username, groups):
        logger.debug("Deleting openfire groups %s from user %s" % (groups, username))
        api = ofUsers(settings.OPENFIRE_ADDRESS, settings.OPENFIRE_SECRET_KEY)
        api.delete_user_groups(username, groups)
        logger.info("Deleted groups %s from openfire user %s" % (groups, username))

    @classmethod
    def send_broadcast_message(cls, group_name, broadcast_message):
        s_group_name = cls._sanitize_groupname(group_name)
        logger.debug("Sending jabber ping to group %s with message %s" % (s_group_name, broadcast_message))
        to_address = s_group_name + '@' + settings.BROADCAST_SERVICE_NAME + '.' + settings.JABBER_URL
        xmpp = PingBot(settings.BROADCAST_USER, settings.BROADCAST_USER_PASSWORD, to_address, broadcast_message)
        xmpp.register_plugin('xep_0030')  # Service Discovery
        xmpp.register_plugin('xep_0199')  # XMPP Ping
        if xmpp.connect(reattempt=False):
            xmpp.process(block=True)
            message = None
            if xmpp.message_sent:
                logger.debug("Sent jabber ping to group %s" % group_name)
                return
            else:
                message = "Failed to send Openfire broadcast message."
            logger.error(message)
            raise PingBotException(message)
        else:
            logger.error("Unable to connect to jabber server")
            raise PingBotException("Unable to connect to jabber server.")


class PingBot(sleekxmpp.ClientXMPP):
    """
    A copy-paste of the example client bot from
    http://sleekxmpp.com/getting_started/sendlogout.html
    """
    def __init__(self, jid, password, recipient, message):
        sleekxmpp.ClientXMPP.__init__(self, jid, password)

        self.reconnect_max_attempts = 5
        self.auto_reconnect = False
        # The message we wish to send, and the JID that
        # will receive it.
        self.recipient = recipient
        self.msg = message

        # Success checking
        self.message_sent = False

        # The session_start event will be triggered when
        # the bot establishes its connection with the server
        # and the XML streams are ready for use. We want to
        # listen for this event so that we we can initialize
        # our roster.
        self.add_event_handler("session_start", self.start)
        if getattr(settings, 'BROADCAST_IGNORE_INVALID_CERT', False):
            self.add_event_handler("ssl_invalid_cert", self.discard)

    def discard(self, *args, **kwargs):
        # Discard the event
        return

    def start(self, event):
        self.send_presence()
        self.get_roster()

        self.send_message(mto=self.recipient,
                          mbody=self.msg,
                          mtype='chat')
        self.message_sent = True
        # Using wait=True ensures that the send queue will be
        # emptied before ending the session.
        self.disconnect(wait=True)


class PingBotException(Exception):
    pass



from django import forms
from django.utils.translation import ugettext_lazy as _


class HRApplicationCommentForm(forms.Form):
    comment = forms.CharField(widget=forms.Textarea, required=False, label=_("Comment"))


class HRApplicationSearchForm(forms.Form):
    search_string = forms.CharField(max_length=254, required=True, label=_("Search String"))



import logging

from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from celery import shared_task
from allianceauth.services.tasks import QueueOnce
from allianceauth.notifications import notify
from allianceauth.services.hooks import NameFormatter
from .manager import SeatManager
from .models import SeatUser

logger = logging.getLogger(__name__)


class SeatTasks:
    def __init__(self):
        pass

    @staticmethod
    def has_account(user):
        try:
            return user.seat.username != ''
        except ObjectDoesNotExist:
            return False

    @classmethod
    def delete_user(cls, user, notify_user=False):
        if cls.has_account(user) and SeatManager.disable_user(user.seat.username):
            user.seat.delete()
            logger.info("Successfully deactivated SeAT for user %s" % user)
            if notify_user:
                notify(user, 'SeAT Account Disabled', level='danger')
            return True
        return False

    @staticmethod
    @shared_task(bind=True, name='seat.update_roles', base=QueueOnce)
    def update_roles(self, pk):
        user = User.objects.get(pk=pk)
        logger.debug("Updating SeAT roles for user %s" % user)
        if SeatTasks.has_account(user):
            groups = [user.profile.state.name]
            for group in user.groups.all():
                groups.append(str(group.name))
            logger.debug("Updating user %s SeAT roles to %s" % (user, groups))
            try:
                SeatManager.update_roles(user.seat.username, groups)
            except:
                logger.warn("SeAT group sync failed for %s, retrying in 10 mins" % user, exc_info=True)
                raise self.retry(countdown=60 * 10)
            logger.debug("Updated user %s SeAT roles." % user)
        else:
            logger.debug("User %s does not have a SeAT account")

    @staticmethod
    @shared_task
    def update_all_roles():
        logger.debug("Updating ALL SeAT roles")
        for user in SeatUser.objects.all():
            SeatTasks.update_roles.delay(user.user_id)

    @staticmethod
    def deactivate():
        SeatUser.objects.all().delete()

    @staticmethod
    def get_username(user):
        from .auth_hooks import SeatService
        return NameFormatter(SeatService(), user).format_name()



from django.contrib import admin
from .models import MumbleUser


class MumbleUserAdmin(admin.ModelAdmin):
        fields = ('user', 'username', 'groups')  # pwhash is hidden from admin panel
        list_display = ('user', 'username', 'groups')
        search_fields = ('user__username', 'username', 'groups')

admin.site.register(MumbleUser, MumbleUserAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-12-04 10:25
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
from django.core.exceptions import ObjectDoesNotExist
import django.db.models.deletion


def internal_group(group):
    return (
        "Corp_" in group.name or
        "Alliance_" in group.name or
        settings.DEFAULT_AUTH_GROUP in group.name or
        settings.DEFAULT_BLUE_GROUP in group.name
    )


def combine_group_models(apps, schema_editor):
    Group = apps.get_model("auth", "Group")
    AuthGroup = apps.get_model("groupmanagement", "AuthGroup")

    for group in Group.objects.all():
        authgroup = AuthGroup(group=group)
        if not hasattr(group, 'hiddengroup'):
            authgroup.hidden = False
        if hasattr(group, 'opengroup'):
            authgroup.open = True

        if hasattr(group, 'groupdescription'):
            authgroup.description = group.groupdescription.description

        authgroup.internal = internal_group(group)
        authgroup.save()


def reverse_group_models(apps, schema_editor):
    Group = apps.get_model("auth", "Group")
    GroupDescription = apps.get_model("groupmanagement", "GroupDescription")
    OpenGroup = apps.get_model("groupmanagement", "OpenGroup")
    HiddenGroup = apps.get_model("groupmanagement", "HiddenGroup")

    for group in Group.objects.all():
        if not hasattr(group, 'authgroup') or group.authgroup is None:
            continue
        if group.authgroup.open:
            OpenGroup.objects.get_or_create(group=group)
        else:
            try:
                OpenGroup.objects.get(group=group).delete()
            except ObjectDoesNotExist:
                pass

        if group.authgroup.hidden:
            HiddenGroup.objects.get_or_create(group=group)
        else:
            try:
                HiddenGroup.objects.get(group=group).delete()
            except ObjectDoesNotExist:
                pass

        if len(group.authgroup.description):
            GroupDescription.objects.update_or_create(group=group,
                                                      defaults={'description': group.authgroup.description})


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('groupmanagement', '0003_default_groups'),
    ]

    operations = [
        migrations.CreateModel(
            name='AuthGroup',
            fields=[
                ('group', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True,
                                               serialize=False,  to='auth.Group')),
                ('internal', models.BooleanField(default=True, help_text='Internal group, users cannot see, join or request to join this group.<br>Used for groups such as Members, Corp_*, Alliance_* etc.<br><b>Overrides Hidden and Open options when selected.</b>')),
                ('hidden', models.BooleanField(default=True, help_text='Group is hidden from users but can still join with the correct link.')),
                ('open', models.BooleanField(default=False, help_text='Group is open and users will be automatically added upon request. <br>If the group is not open users will need their request manually approved.')),
                ('description', models.CharField(max_length=512, blank=True,  help_text='Description of the group shown to users.', )),

                ('group_leaders', models.ManyToManyField(related_name='leads_groups', to=settings.AUTH_USER_MODEL, blank=True, help_text='Group leaders can process group requests for this group specifically. Use the auth.group_management permission to allow a user to manage all groups.',)),
            ],
        ),
        migrations.RunPython(combine_group_models, reverse_group_models),
        migrations.RemoveField(
            model_name='groupdescription',
            name='group',
        ),
        migrations.RemoveField(
            model_name='hiddengroup',
            name='group',
        ),
        migrations.RemoveField(
            model_name='opengroup',
            name='group',
        ),
        migrations.DeleteModel(
            name='GroupDescription',
        ),
        migrations.DeleteModel(
            name='HiddenGroup',
        ),
        migrations.DeleteModel(
            name='OpenGroup',
        ),
    ]



import logging

from django.template.loader import render_to_string
from django.conf import settings

from allianceauth import hooks
from allianceauth.services.hooks import ServicesHook
from .tasks import DiscourseTasks
from .urls import urlpatterns

logger = logging.getLogger(__name__)


class DiscourseService(ServicesHook):
    def __init__(self):
        ServicesHook.__init__(self)
        self.urlpatterns = urlpatterns
        self.name = 'discourse'
        self.service_ctrl_template = 'services/discourse/discourse_service_ctrl.html'
        self.access_perm = 'discourse.access_discourse'
        self.name_format = '{character_name}'

    def delete_user(self, user, notify_user=False):
        logger.debug('Deleting user %s %s account' % (user, self.name))
        return DiscourseTasks.delete_user(user, notify_user=notify_user)

    def update_groups(self, user):
        logger.debug('Processing %s groups for %s' % (self.name, user))
        if DiscourseTasks.has_account(user):
            DiscourseTasks.update_groups.delay(user.pk)

    def validate_user(self, user):
        logger.debug('Validating user %s %s account' % (user, self.name))
        if DiscourseTasks.has_account(user) and not self.service_active_for_user(user):
            self.delete_user(user, notify_user=True)

    def update_all_groups(self):
        logger.debug('Update all %s groups called' % self.name)
        DiscourseTasks.update_all_groups.delay()

    def service_active_for_user(self, user):
        return user.has_perm(self.access_perm)

    def render_services_ctrl(self, request):
        return render_to_string(self.service_ctrl_template, {
            'char': request.user.profile.main_character,
            'DISCOURSE_URL': getattr(settings, 'DISCOURSE_URL', ''),
        }, request=request)


@hooks.register('services_hook')
def register_service():
    return DiscourseService()



import hashlib
import logging
import random
import string

import requests
from django.conf import settings

logger = logging.getLogger(__name__)


class SeatManager:
    def __init__(self):
        pass

    RESPONSE_OK = 'ok'

    @staticmethod
    def __sanitize_username(username):
        sanatized = username.replace(" ", "_")
        return sanatized.lower()

    @staticmethod
    def __generate_random_pass():
        return ''.join([random.choice(string.ascii_letters + string.digits) for n in range(16)])

    @classmethod
    def _response_ok(cls, response):
        return cls.RESPONSE_OK in response

    @staticmethod
    def exec_request(endpoint, func, raise_for_status=False, **kwargs):
        """ Send an https api request """
        try:
            endpoint = '{0}/api/v1/{1}'.format(settings.SEAT_URL, endpoint)
            headers = {'X-Token': settings.SEAT_XTOKEN, 'Accept': 'application/json'}
            logger.debug(headers)
            logger.debug(endpoint)
            ret = getattr(requests, func)(endpoint, headers=headers, data=kwargs)
            ret.raise_for_status()
            return ret.json()
        except requests.HTTPError as e:
            if raise_for_status:
                raise e
            logger.exception("Error encountered while performing API request to SeAT with url {}".format(endpoint))
            return {}

    @classmethod
    def add_user(cls, username, email):
        """ Add user to service """
        sanitized = str(cls.__sanitize_username(username))
        logger.debug("Adding user to SeAT with username %s" % sanitized)
        password = cls.__generate_random_pass()
        ret = cls.exec_request('user', 'post', username=sanitized, email=str(email), password=password)
        logger.debug(ret)
        if cls._response_ok(ret):
            logger.info("Added SeAT user with username %s" % sanitized)
            return sanitized, password
        logger.info("Failed to add SeAT user with username %s" % sanitized)
        return None, None

    @classmethod
    def delete_user(cls, username):
        """ Delete user """
        ret = cls.exec_request('user/{}'.format(username), 'delete')
        logger.debug(ret)
        if cls._response_ok(ret):
            logger.info("Deleted SeAT user with username %s" % username)
            return username
        return None

    @classmethod
    def enable_user(cls, username):
        """ Enable user """
        ret = cls.exec_request('user/{}'.format(username), 'put', account_status=1)
        logger.debug(ret)
        if cls._response_ok(ret):
            logger.info("Enabled SeAT user with username %s" % username)
            return username
        logger.info("Failed to enabled SeAT user with username %s" % username)
        return None

    @classmethod
    def disable_user(cls, username):
        """ Disable user """
        cls.update_roles(username, [])
        ret = cls.exec_request('user/{}'.format(username), 'put', account_status=0)
        logger.debug(ret)
        if cls._response_ok(ret):
            logger.info("Disabled SeAT user with username %s" % username)
            return username
        logger.info("Failed to disable SeAT user with username %s" % username)
        return None

    @classmethod
    def _check_email_changed(cls, username, email):
        """Compares email to one set on SeAT"""
        ret = cls.exec_request('user/{}'.format(username), 'get', raise_for_status=True)
        return ret['email'] != email

    @classmethod
    def update_user(cls, username, email, password):
        """ Edit user info """
        if cls._check_email_changed(username, email):
            # if we try to set the email to whatever it is already on SeAT, we get a HTTP422 error
            logger.debug("Updating SeAT username %s with email %s and password" % (username, email))
            ret = cls.exec_request('user/{}'.format(username), 'put', email=email)
            logger.debug(ret)
            if not cls._response_ok(ret):
                logger.warn("Failed to update email for username {}".format(username))
        ret = cls.exec_request('user/{}'.format(username), 'put', password=password)
        logger.debug(ret)
        if not cls._response_ok(ret):
            logger.warn("Failed to update password for username {}".format(username))
            return None
        logger.info("Updated SeAT user with username %s" % username)
        return username

    @classmethod
    def update_user_password(cls, username, email, plain_password=None):
        logger.debug("Settings new SeAT password for user %s" % username)
        if not plain_password:
            plain_password = cls.__generate_random_pass()
        if cls.update_user(username, email, plain_password):
            return plain_password

    @classmethod
    def check_user_status(cls, username):
        sanitized = str(cls.__sanitize_username(username))
        logger.debug("Checking SeAT status for user %s" % sanitized)
        ret = cls.exec_request('user/{}'.format(sanitized), 'get')
        logger.debug(ret)
        return ret

    @classmethod
    def get_all_seat_eveapis(cls):
        seat_all_keys = cls.exec_request('key', 'get')
        seat_keys = {}
        for key in seat_all_keys:
            try:
                seat_keys[key["key_id"]] = key["user_id"]
            except KeyError:
                seat_keys[key["key_id"]] = None
        return seat_keys

    @classmethod
    def get_all_roles(cls):
        groups = {}
        ret = cls.exec_request('role', 'get')
        logger.debug(ret)
        for group in ret:
            groups[group["title"]] = group["id"]
        logger.debug("Retrieved role list from SeAT: %s" % str(groups))
        return groups

    @classmethod
    def add_role(cls, role):
        ret = cls.exec_request('role/new', 'post', name=role)
        logger.debug(ret)
        logger.info("Added Seat group %s" % role)
        role_info = cls.exec_request('role/detail/{}'.format(role), 'get')
        logger.debug(role_info)
        return role_info["id"]

    @classmethod
    def add_role_to_user(cls, user_id, role_id):
        ret = cls.exec_request('role/grant-user-role/{}/{}'.format(user_id, role_id), 'get')
        logger.info("Added role %s to user %s" % (role_id, user_id))
        return ret

    @classmethod
    def revoke_role_from_user(cls, user_id, role_id):
        ret = cls.exec_request('role/revoke-user-role/{}/{}'.format(user_id, role_id), 'get')
        logger.info("Revoked role %s from user %s" % (role_id, user_id))
        return ret

    @classmethod
    def update_roles(cls, seat_user, roles):
        logger.debug("Updating SeAT user %s with roles %s" % (seat_user, roles))
        user_info = cls.check_user_status(seat_user)
        user_roles = {}
        if type(user_info["roles"]) is list:
            for role in user_info["roles"]:
                user_roles[role["title"]] = role["id"]
        logger.debug("Got user %s SeAT roles %s" % (seat_user, user_roles))
        seat_roles = cls.get_all_roles()
        addroles = set(roles) - set(user_roles.keys())
        remroles = set(user_roles.keys()) - set(roles)

        logger.info("Updating SeAT roles for user %s - adding %s, removing %s" % (seat_user, addroles, remroles))
        for r in addroles:
            if r not in seat_roles:
                seat_roles[r] = cls.add_role(r)
            logger.debug("Adding role %s to SeAT user %s" % (r, seat_user))
            cls.add_role_to_user(user_info["id"], seat_roles[r])
        for r in remroles:
            logger.debug("Removing role %s from user %s" % (r, seat_user))
            cls.revoke_role_from_user(user_info["id"], seat_roles[r])

    @staticmethod
    def username_hash(username):
        m = hashlib.sha1()
        m.update(username.encode('utf-8'))
        return m.hexdigest()



from django_webtest import WebTest
from django.utils import timezone
from django.urls import reverse
from django.contrib.auth.models import Permission, User
from django.conf import settings

from datetime import timedelta

from allianceauth.tests.auth_utils import AuthUtils
from allianceauth.eveonline.models import EveCorporationInfo

from .models import Timer
from .form import TimerForm


class TimerboardViewsTestCase(WebTest):
    csrf_checks = False

    def setUp(self):
        corp = EveCorporationInfo.objects.create(corporation_id='2345', corporation_name='test corp',
                                                 corporation_ticker='testc', member_count=24)
        other_corp = EveCorporationInfo.objects.create(corporation_id='9345', corporation_name='other test corp',
                                                       corporation_ticker='testd', member_count=1)
        self.user = AuthUtils.create_user('test_user')
        AuthUtils.add_main_character(self.user, 'test character', '1234', '2345', 'test corp', 'testc')
        self.user = User.objects.get_by_natural_key('test_user')
        character = self.user.profile.main_character
        self.other_user = AuthUtils.create_user('other_test_user')
        AuthUtils.add_main_character(self.other_user, 'test character 2', '9234', '9345', 'other test corp', 'testd')
        self.other_user = User.objects.get_by_natural_key('other_test_user')
        other_character = self.other_user.profile.main_character

        self.timer = Timer.objects.create(
            details='details',
            system='system',
            planet_moon='planet_moon',
            structure='structure',
            objective='objective',
            eve_time=timezone.now() + timedelta(days=30),
            important=True,
            corp_timer=False,
            eve_character=character,
            eve_corp=character.corporation,
            user=self.user,
        )
        self.corp_timer = Timer.objects.create(
            details='details',
            system='system',
            planet_moon='planet_moon',
            structure='structure',
            objective='objective',
            eve_time=timezone.now() + timedelta(days=30),
            important=False,
            corp_timer=True,
            eve_character=character,
            eve_corp=character.corporation,
            user=self.user,
        )
        self.other_corp_timer = Timer.objects.create(
            details='details',
            system='system',
            planet_moon='planet_moon',
            structure='structure',
            objective='objective',
            eve_time=timezone.now() + timedelta(days=30),
            important=False,
            corp_timer=True,
            eve_character=other_character,
            eve_corp=other_character.corporation,
            user=self.user,
        )
        self.expired_timer = Timer.objects.create(
            details='details',
            system='system',
            planet_moon='planet_moon',
            structure='structure',
            objective='objective',
            eve_time=timezone.now() - timedelta(days=30),
            important=True,
            corp_timer=False,
            eve_character=character,
            eve_corp=character.corporation,
            user=self.user,
        )

        self.view_permission = Permission.objects.get(codename='timer_view')
        self.edit_permission = Permission.objects.get(codename='timer_management')

        self.view_url = reverse('timerboard:view')
        self.add_url = reverse('timerboard:add')
        self.edit_url_name = 'timerboard:edit'
        self.delete_url_name = 'timerboard:delete'

    def test_timer_view(self):
        self.user.user_permissions.add(self.view_permission)

        self.app.set_user(self.user)

        response = self.app.get(self.view_url)

        context = response.context[-1]

        timers = context['timers']
        corp_timers = context['corp_timers']
        future_timers = context['future_timers']
        past_timers = context['past_timers']

        self.assertTemplateUsed(response, 'timerboard/view.html')

        self.assertIn(self.timer, timers)
        self.assertIn(self.expired_timer, timers)
        self.assertNotIn(self.corp_timer, timers)
        self.assertNotIn(self.other_corp_timer, timers)

        self.assertNotIn(self.timer, corp_timers)
        self.assertNotIn(self.expired_timer, corp_timers)
        self.assertIn(self.corp_timer, corp_timers)
        self.assertNotIn(self.other_corp_timer, corp_timers)

        self.assertIn(self.timer, future_timers)
        self.assertNotIn(self.expired_timer, future_timers)
        self.assertNotIn(self.corp_timer, future_timers)
        self.assertNotIn(self.other_corp_timer, future_timers)

        self.assertNotIn(self.timer, past_timers)
        self.assertIn(self.expired_timer, past_timers)
        self.assertNotIn(self.corp_timer, past_timers)
        self.assertNotIn(self.other_corp_timer, past_timers)

    def test_timer_view_permission(self):
        self.client.force_login(self.user)
        response = self.app.get(self.view_url)
        self.assertRedirects(response, expected_url=reverse(settings.LOGIN_URL) + '?next=' + self.view_url)

    def test_timer_view_login(self):
        response = self.app.get(self.view_url)
        self.assertRedirects(response, expected_url=reverse(settings.LOGIN_URL) + '?next=' + self.view_url)

    def test_add_timer_get(self):
        self.user.user_permissions.add(self.edit_permission)
        self.app.set_user(self.user)

        response = self.app.get(self.add_url)

        self.assertTemplateUsed(response, 'timerboard/timer_create_form.html')

        context = response.context[-1]

        self.assertIs(TimerForm, type(context['form']))

    def test_add_timer_form_error(self):
        self.user.user_permissions.add(self.edit_permission)
        self.app.set_user(self.user)
        page = self.app.post(self.add_url)
        page = page.forms['add-timer-form'].submit()
        self.assertContains(page, "This field is required.")

    def test_add_timer_post(self):
        self.user.user_permissions.add(self.edit_permission)
        self.user.user_permissions.add(self.view_permission)

        self.app.set_user(self.user)

        page = self.app.post(self.add_url)
        form = page.forms['add-timer-form']

        form['details'] = 'details'
        form['system'] = 'jita'
        form['planet_moon'] = '4-4'
        form['structure'] = TimerForm.structure_choices[0][0]
        form['objective'] = TimerForm.objective_choices[0][0]
        form['days_left'] = 1
        form['hours_left'] = 2
        form['minutes_left'] = 3
        form['important'] = True
        form['corp_timer'] = False

        response = form.submit()

        self.assertRedirects(response, self.view_url)

        self.assertTrue(Timer.objects.filter(system='jita', details='details').exists())

    def test_edit_timer_get(self):
        self.user.user_permissions.add(self.edit_permission)
        self.app.set_user(self.user)

        response = self.app.get(reverse(self.edit_url_name, args=[self.timer.id]))
        context = response.context[-1]
        form = context['form']
        data = form.instance

        self.assertTemplateUsed(response, 'timerboard/timer_update_form.html')
        self.assertIs(TimerForm, type(form))
        self.assertEqual(data, self.timer)

    def test_edit_timer_post(self):
        self.user.user_permissions.add(self.edit_permission)
        self.user.user_permissions.add(self.view_permission)

        self.app.set_user(self.user)

        page = self.app.post(reverse(self.edit_url_name, args=[self.timer.id]))
        form = page.forms['add-timer-form']

        form['details'] = 'detailsUNIQUE'
        form['system'] = 'jita'
        form['planet_moon'] = '4-4'
        form['structure'] = TimerForm.structure_choices[0][0]
        form['objective'] = TimerForm.objective_choices[0][0]
        form['days_left'] = 1
        form['hours_left'] = 2
        form['minutes_left'] = 3
        form['important'] = True
        form['corp_timer'] = False

        response = form.submit()

        self.assertRedirects(response, self.view_url)

        self.assertTrue(Timer.objects.filter(system='jita', details='detailsUNIQUE').exists())

    def test_delete_timer_get(self):
        self.user.user_permissions.add(self.edit_permission)
        self.app.set_user(self.user)

        response = self.app.get(reverse(self.delete_url_name, args=[self.timer.id]))

        self.assertTemplateUsed(response, 'timerboard/timer_confirm_delete.html')

        self.assertContains(response, 'Are you sure you want to delete timer "'+str(self.timer))



from allianceauth import NAME

import requests
import logging

logger = logging.getLogger(__name__)


class SRPManager:
    def __init__(self):
        pass

    @staticmethod
    def get_kill_id(killboard_link):
        num_set = '0123456789'
        kill_id = ''.join([c for c in killboard_link if c in num_set])
        return kill_id

    @staticmethod
    def get_kill_data(kill_id):
        url = ("https://www.zkillboard.com/api/killID/%s/" % kill_id)
        headers = {
            'User-Agent': NAME,
            'Content-Type': 'application/json',
        }
        r = requests.get(url, headers=headers)
        result = r.json()[0]
        if result:
            ship_type = result['victim']['ship_type_id']
            logger.debug("Ship type for kill ID %s is %s" % (kill_id, ship_type))
            ship_value = result['zkb']['totalValue']
            logger.debug("Total loss value for kill id %s is %s" % (kill_id, ship_value))
            victim_id = result['victim']['character_id']
            return ship_type, ship_value, victim_id
        else:
            raise ValueError("Invalid Kill ID")




# Generated by Django 2.0.4 on 2018-04-14 18:28

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


def create_initial_records(apps, schema_editor):
    OwnershipRecord = apps.get_model('authentication', 'OwnershipRecord')
    CharacterOwnership = apps.get_model('authentication', 'CharacterOwnership')

    OwnershipRecord.objects.bulk_create([
        OwnershipRecord(user=o.user, character=o.character, owner_hash=o.owner_hash) for o in CharacterOwnership.objects.all()
    ])


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('eveonline', '0009_on_delete'),
        ('authentication', '0015_user_profiles'),
    ]

    operations = [
        migrations.CreateModel(
            name='OwnershipRecord',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('owner_hash', models.CharField(db_index=True, max_length=28)),
                ('created', models.DateTimeField(auto_now=True)),
                ('character', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ownership_records', to='eveonline.EveCharacter')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ownership_records', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['-created'],
            },
        ),
        migrations.RunPython(create_initial_records, migrations.RunPython.noop)
    ]



import logging

from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import render, redirect
from django.conf import settings
from .manager import Teamspeak3Manager
from .forms import TeamspeakJoinForm
from .models import Teamspeak3User
from .tasks import Teamspeak3Tasks

logger = logging.getLogger(__name__)

ACCESS_PERM = 'teamspeak3.access_teamspeak3'


@login_required
@permission_required(ACCESS_PERM)
def activate_teamspeak3(request):
    logger.debug("activate_teamspeak3 called by user %s" % request.user)

    character = request.user.profile.main_character
    with Teamspeak3Manager() as ts3man:
        logger.debug("Adding TS3 user for user %s with main character %s" % (request.user, character))
        result = ts3man.add_user(request.user, Teamspeak3Tasks.get_username(request.user))

    # if its empty we failed
    if result[0] is not "":
        Teamspeak3User.objects.update_or_create(user=request.user, defaults={'uid': result[0], 'perm_key': result[1]})
        logger.debug("Updated authserviceinfo for user %s with TS3 credentials. Updating groups." % request.user)
        logger.info("Successfully activated TS3 for user %s" % request.user)
        messages.success(request, 'Activated TeamSpeak3 account.')
        return redirect("teamspeak3:verify")
    logger.error("Unsuccessful attempt to activate TS3 for user %s" % request.user)
    messages.error(request, 'An error occurred while processing your TeamSpeak3 account.')
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def verify_teamspeak3(request):
    logger.debug("verify_teamspeak3 called by user %s" % request.user)
    if not Teamspeak3Tasks.has_account(request.user):
        logger.warn("Unable to validate user %s teamspeak: no teamspeak data" % request.user)
        return redirect("services:services")
    if request.method == "POST":
        form = TeamspeakJoinForm(request.POST)
        if form.is_valid():
            Teamspeak3Tasks.update_groups.delay(request.user.pk)
            logger.debug("Validated user %s joined TS server" % request.user)
            return redirect("services:services")
    else:
        form = TeamspeakJoinForm(initial={'username': request.user.teamspeak3.uid})
    context = {
        'form': form,
        'authinfo': {'teamspeak3_uid': request.user.teamspeak3.uid,
                     'teamspeak3_perm_key': request.user.teamspeak3.perm_key},
        'public_url': settings.TEAMSPEAK3_PUBLIC_URL,
    }
    return render(request, 'services/teamspeak3/teamspeakjoin.html', context=context)


@login_required
@permission_required(ACCESS_PERM)
def deactivate_teamspeak3(request):
    logger.debug("deactivate_teamspeak3 called by user %s" % request.user)
    if Teamspeak3Tasks.has_account(request.user) and Teamspeak3Tasks.delete_user(request.user):
        logger.info("Successfully deactivated TS3 for user %s" % request.user)
        messages.success(request, 'Deactivated TeamSpeak3 account.')
    else:
        logger.error("Unsuccessful attempt to deactivate TS3 for user %s" % request.user)
        messages.error(request, 'An error occurred while processing your TeamSpeak3 account.')
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def reset_teamspeak3_perm(request):
    logger.debug("reset_teamspeak3_perm called by user %s" % request.user)
    if not Teamspeak3Tasks.has_account(request.user):
        return redirect("services:services")
    logger.debug("Deleting TS3 user for user %s" % request.user)
    with Teamspeak3Manager() as ts3man:
        ts3man.delete_user(request.user.teamspeak3.uid)

        logger.debug("Generating new permission key for user %s" % request.user)
        result = ts3man.generate_new_permissionkey(request.user.teamspeak3.uid, request.user, Teamspeak3Tasks.get_username(request.user))

    # if blank we failed
    if result[0] != "":
        Teamspeak3User.objects.update_or_create(user=request.user, defaults={'uid': result[0], 'perm_key': result[1]})
        logger.debug("Updated authserviceinfo for user %s with TS3 credentials. Updating groups." % request.user)
        Teamspeak3Tasks.update_groups.delay(request.user.pk)
        logger.info("Successfully reset TS3 permission key for user %s" % request.user)
        messages.success(request, 'Reset TeamSpeak3 permission key.')
    else:
        logger.error("Unsuccessful attempt to reset TS3 permission key for user %s" % request.user)
        messages.error(request, 'An error occurred while processing your TeamSpeak3 account.')
    return redirect("services:services")



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-12 13:04
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('authentication', '0007_remove_authservicesinfo_is_blue'),
        ('eveonline', '0001_initial'),
        ('auth', '0001_initial'),
    ]

    operations = [
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-09 23:19
from __future__ import unicode_literals

from django.db import migrations

def create_permissions(apps, schema_editor):
    User = apps.get_model('auth', 'User')
    ContentType = apps.get_model('contenttypes', 'ContentType')
    Permission = apps.get_model('auth', 'Permission')
    ct = ContentType.objects.get_for_model(User)
    Permission.objects.get_or_create(codename="member", content_type=ct, name="member")
    Permission.objects.get_or_create(codename="group_management", content_type=ct, name="group_management")
    Permission.objects.get_or_create(codename="jabber_broadcast", content_type=ct, name="jabber_broadcast")
    Permission.objects.get_or_create(codename="jabber_broadcast_all", content_type=ct, name="jabber_broadcast_all")
    Permission.objects.get_or_create(codename="fleetactivitytracking", content_type=ct, name="fleetactivitytracking")
    Permission.objects.get_or_create(codename="fleetactivitytracking_statistics", content_type=ct, name="fleetactivitytracking_statistics")
    Permission.objects.get_or_create(codename="human_resources", content_type=ct, name="human_resources")
    Permission.objects.get_or_create(codename="blue_member", content_type=ct, name="blue_member")
    Permission.objects.get_or_create(codename="alliance_apis", content_type=ct, name="alliance_apis")
    Permission.objects.get_or_create(codename="corp_apis", content_type=ct, name="corp_apis")
    Permission.objects.get_or_create(codename="timer_management", content_type=ct, name="timer_management")
    Permission.objects.get_or_create(codename="timer_view", content_type=ct, name="timer_view")
    Permission.objects.get_or_create(codename="srp_management", content_type=ct, name="srp_management")
    Permission.objects.get_or_create(codename="signature_management", content_type=ct, name="signature_management")
    Permission.objects.get_or_create(codename="signature_view", content_type=ct, name="signature_view")
    Permission.objects.get_or_create(codename="optimer_management", content_type=ct, name="optimer_management")
    Permission.objects.get_or_create(codename="optimer_view", content_type=ct, name="optimer_view")
    Permission.objects.get_or_create(codename="logging_notifications", content_type=ct, name="logging_notifications")

def reverse(apps, schema_editor):
    #too lazy
    pass

class Migration(migrations.Migration):

    dependencies = [
        ('authentication', '0003_authservicesinfo_state'),
    ]

    operations = [
        migrations.RunPython(create_permissions, reverse)
    ]



from allianceauth.services.hooks import MenuItemHook, UrlHook
from django.utils.translation import ugettext_lazy as _
from allianceauth import hooks
from allianceauth.hrapplications import urls


class ApplicationsMenu(MenuItemHook):
    def __init__(self):
        MenuItemHook.__init__(self,
                              _('Applications'),
                              'fa fa-file-o fa-fw',
                              'hrapplications:index',
                              navactive=['hrapplications:'])


@hooks.register('menu_item_hook')
def register_menu():
    return ApplicationsMenu()


class ApplicationsUrls(UrlHook):
    def __init__(self):
        UrlHook.__init__(self, urls, 'hrapplications', r'^hr/')


@hooks.register('url_hook')
def register_url():
    return ApplicationsUrls()



from django.db import models


class PermissionsTool(models.Model):
    """
    Dummy model for holding permissions
    """
    class Meta:
        managed = False
        permissions = (
            ('audit_permissions', 'Can audit permissions'),
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2017-01-02 19:23
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0005_remove_eveallianceinfo_member_count'),
    ]

    operations = [
        migrations.AlterField(
            model_name='evecharacter',
            name='alliance_id',
            field=models.CharField(blank=True, default='', max_length=254, null=True),
        ),
        migrations.AlterField(
            model_name='evecharacter',
            name='alliance_name',
            field=models.CharField(blank=True, default='', max_length=254, null=True),
        ),
    ]



from django.contrib.auth.models import User
from django.db import models


class ExampleUser(models.Model):
    user = models.OneToOneField(User,
                                primary_key=True,
                                on_delete=models.CASCADE,
                                related_name='example')
    username = models.CharField(max_length=254)

    def __str__(self):
        return self.username



from django.db import models

from allianceauth.authentication.models import State


class NameFormatConfig(models.Model):
    service_name = models.CharField(max_length=100, blank=False, null=False)
    default_to_username = models.BooleanField(default=True, help_text="If a user has no main_character, "
                                                                      "default to using their Auth username instead.")
    format = models.CharField(max_length=100, blank=False, null=False,
                              help_text='For information on constructing name formats, please see the '
                                        '<a href="https://allianceauth.readthedocs.io/en/latest/features/nameformats">'
                                        'name format documentation</a>')
    states = models.ManyToManyField(State, help_text="States to apply this format to. You should only have one "
                                                     "formatter for each state for each service.")





from django import forms
from django.utils.translation import ugettext_lazy as _


class FleetFormatterForm(forms.Form):
    fleet_name = forms.CharField(label=_('Name of Fleet:'), required=True)
    fleet_commander = forms.CharField(label=_('Fleet Commander:'), required=True)
    fleet_comms = forms.CharField(label=_('Fleet Comms:'), required=True)
    fleet_type = forms.CharField(label=_('Fleet Type:'), required=True)
    ship_priorities = forms.CharField(label=_('Ship Priorities:'), required=True)
    formup_location = forms.CharField(label=_('Formup Location:'), required=True)
    formup_time = forms.CharField(label=_('Formup Time:'), required=True)
    expected_duration = forms.CharField(label=_('Expected Duration:'), required=True)
    purpose = forms.CharField(label=_('Purpose:'), required=True)
    reimbursable = forms.ChoiceField(label=_('Reimbursable?*'), choices=[(_('Yes'), _('Yes')), (_('No'), _('No'))],
                                     required=True)
    important = forms.ChoiceField(label=_('Important?*'), choices=[(_('Yes'), _('Yes')), (_('No'), _('No'))],
                                  required=True)
    comments = forms.CharField(label=_('Comments'), widget=forms.Textarea, required=False)


class ServicePasswordForm(forms.Form):
    password = forms.CharField(label=_("Password"), required=True, widget=forms.PasswordInput())

    def clean_password(self):
        password = self.cleaned_data['password']
        if not len(password) >= 8:
            raise forms.ValidationError(_("Password must be at least 8 characters long."))
        return password


class ServicePasswordModelForm(forms.ModelForm):
    password = forms.CharField(label=_("Password"), required=True, widget=forms.PasswordInput())

    def clean_password(self):
        password = self.cleaned_data['password']
        if not len(password) >= 8:
            raise forms.ValidationError(_("Password must be at least 8 characters long."))
        return password

    def save(self, commit=True):
        svc_obj = super(ServicePasswordModelForm, self).save(commit=False)
        svc_obj.update_password(self.cleaned_data['password'])
        if commit:
            svc_obj.save()
        return svc_obj



import datetime
import logging

from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.contrib.auth.decorators import permission_required
from django.shortcuts import render
from django.template.defaulttags import register
from django.utils.translation import ugettext_lazy as _

from .managers import FleetUpManager

logger = logging.getLogger(__name__)


@register.filter
def get_item(dictionary, key):
    return dictionary.get(key)


@login_required
@permission_required('auth.view_fleetup')
def fleetup_view(request):
    logger.debug("fleetup_view called by user %s" % request.user)

    operations_list = FleetUpManager.get_fleetup_operations()
    if operations_list is None:
        messages.add_message(request, messages.ERROR, _("Failed to get operations list, contact your administrator"))
        operations_list = {}
    timers_list = FleetUpManager.get_fleetup_timers()
    if timers_list is None:
        messages.add_message(request, messages.ERROR, _("Failed to get timers list, contact your administrator"))
        timers_list = {}
    now = datetime.datetime.now().strftime('%H:%M:%S')

    context = {"timers_list": sorted(timers_list.items()),
               "operations_list": sorted(operations_list.items()),
               "now": now}

    return render(request, 'fleetup/index.html', context=context)


@login_required
@permission_required('auth.human_resources')
@permission_required('auth.view_fleetup')
def fleetup_characters(request):
    logger.debug("fleetup_characters called by user %s" % request.user)

    member_list = FleetUpManager.get_fleetup_members()
    if member_list is None:
        messages.add_message(request, messages.ERROR, _("Failed to get member list, contact your administrator"))
        member_list = {}

    context = {"member_list": sorted(member_list.items())}

    return render(request, 'fleetup/characters.html', context=context)


@login_required
@permission_required('auth.view_fleetup')
def fleetup_fittings(request):
    logger.debug("fleetup_fittings called by user %s" % request.user)
    fitting_list = FleetUpManager.get_fleetup_fittings()

    if fitting_list is None:
        messages.add_message(request, messages.ERROR, _("Failed to get fitting list, contact your administrator"))
        fitting_list = {}

    context = {"fitting_list": sorted(fitting_list.items())}
    return render(request, 'fleetup/fittingsview.html', context=context)


@login_required
@permission_required('auth.view_fleetup')
def fleetup_fitting(request, fittingnumber):
    logger.debug("fleetup_fitting called by user %s" % request.user)
    fitting_eft = FleetUpManager.get_fleetup_fitting_eft(fittingnumber)
    fitting_data = FleetUpManager.get_fleetup_fitting(fittingnumber)
    doctrinenumber = FleetUpManager.get_fleetup_doctrineid(fittingnumber)
    doctrines_list = FleetUpManager.get_fleetup_doctrine(doctrinenumber)

    if fitting_eft is None or fitting_data is None or doctrinenumber is None:
        messages.add_message(request, messages.ERROR, _("There was an error getting some of the data for this fitting. "
                                                        "Contact your administrator"))

    context = {"fitting_eft": fitting_eft,
               "fitting_data": fitting_data,
               "doctrines_list": doctrines_list}
    return render(request, 'fleetup/fitting.html', context=context)


@login_required
@permission_required('auth.view_fleetup')
def fleetup_doctrines(request):
    logger.debug("fleetup_doctrines called by user %s" % request.user)
    doctrines_list = FleetUpManager.get_fleetup_doctrines()
    if doctrines_list is None:
        messages.add_message(request, messages.ERROR, _("Failed to get doctrines list, contact your administrator"))

    context = {"doctrines_list": doctrines_list}
    return render(request, 'fleetup/doctrinesview.html', context=context)


@login_required
@permission_required('auth.view_fleetup')
def fleetup_doctrine(request, doctrinenumber):
    logger.debug("fleetup_doctrine called by user %s" % request.user)
    doctrine = FleetUpManager.get_fleetup_doctrine(doctrinenumber)
    if doctrine is None:
        messages.add_message(request, messages.ERROR, _("Failed to get doctine, contact your administrator"))
    context = {"doctrine": doctrine}
    return render(request, 'fleetup/doctrine.html', context=context)



from django.test import TestCase
from allianceauth.tests.auth_utils import AuthUtils

from ..models import AutogroupsConfig
from . import patch


class AutogroupsConfigManagerTestCase(TestCase):

    def test_update_groups_for_state(self):
        member = AuthUtils.create_member('test member')
        obj = AutogroupsConfig.objects.create()
        obj.states.add(member.profile.state)

        with patch('.models.AutogroupsConfig.update_group_membership_for_user') as update_group_membership_for_user:
            AutogroupsConfig.objects.update_groups_for_state(member.profile.state)

            self.assertTrue(update_group_membership_for_user.called)
            self.assertEqual(update_group_membership_for_user.call_count, 1)
            args, kwargs = update_group_membership_for_user.call_args
            self.assertEqual(args[0], member)

    def test_update_groups_for_user(self):
        member = AuthUtils.create_member('test member')
        obj = AutogroupsConfig.objects.create()
        obj.states.add(member.profile.state)

        with patch('.models.AutogroupsConfig.update_group_membership_for_user') as update_group_membership_for_user:
            AutogroupsConfig.objects.update_groups_for_user(member)

            self.assertTrue(update_group_membership_for_user.called)
            self.assertEqual(update_group_membership_for_user.call_count, 1)
            args, kwargs = update_group_membership_for_user.call_args
            self.assertEqual(args[0], member)

    @patch('.models.AutogroupsConfig.update_group_membership_for_user')
    @patch('.models.AutogroupsConfig.remove_user_from_alliance_groups')
    @patch('.models.AutogroupsConfig.remove_user_from_corp_groups')
    def test_update_groups_no_config(self, remove_corp, remove_alliance, update_groups):
        member = AuthUtils.create_member('test member')
        obj = AutogroupsConfig.objects.create()

        # Corp and alliance groups should be removed from users if their state has no config
        AutogroupsConfig.objects.update_groups_for_user(member)

        self.assertFalse(update_groups.called)
        self.assertTrue(remove_alliance.called)
        self.assertTrue(remove_corp.called)

        # The normal group assignment should occur if there state has a config
        obj.states.add(member.profile.state)
        AutogroupsConfig.objects.update_groups_for_user(member)

        self.assertTrue(update_groups.called)



# -*- coding: utf-8 -*-
from setuptools import setup
import allianceauth

install_requires = [
    'mysqlclient',
    'dnspython',
    'passlib',
    'requests>=2.9.1',
    'bcrypt',
    'python-slugify>=1.2',
    'requests-oauthlib',
    'semantic_version',

    'redis',
    'celery>=4.0.2',
    'celery_once',

    'django>=1.11',
    'django-bootstrap-form',
    'django-registration==2.4',
    'django-sortedm2m',
    'django-redis-cache>=1.7.1',
    'django-celery-beat',

    'openfire-restapi',
    'sleekxmpp',

    'adarnauth-esi>=1.4.10,<2.0',
]

testing_extras = [
    'coverage>=4.3.1',
    'requests-mock>=1.2.0',
    'django-nose',
    'django-webtest',
]

setup(
    name='allianceauth',
    version=allianceauth.__version__,
    author='Alliance Auth',
    author_email='adarnof@gmail.com',
    description='An auth system for EVE Online to help in-game organizations manage online service access.',
    install_requires=install_requires,
    extras_require={
        'testing': testing_extras,
        ':python_version=="3.4"': ['typing'],
    },
    python_requires='~=3.4',
    license='GPLv2',
    packages=['allianceauth'],
    url='https://github.com/allianceauth/allianceauth',
    zip_safe=False,
    include_package_data=True,
    entry_points="""
            [console_scripts]
            allianceauth=allianceauth.bin.allianceauth:main
    """,
)



import logging

from django.conf import settings

from .util.ts3 import TS3Server, TeamspeakError
from .models import TSgroup

logger = logging.getLogger(__name__)


class Teamspeak3Manager:
    def __init__(self):
        self._server = None

    @property
    def server(self):
        if self._server is not None and self._server._connected:
            return self._server
        else:
            raise ValueError("Teamspeak not connected")

    def connect(self):
        self._server = self.__get_created_server()
        return self

    def disconnect(self):
        self._server.disconnect()
        self._server = None

    def __enter__(self):
        logger.debug("Entering with statement, connecting")
        self.connect()
        return self

    def __exit__(self, _type, value, traceback):
        logger.debug("Exiting with statement, cleaning up")
        self.disconnect()

    @staticmethod
    def __get_created_server():
        server = TS3Server(settings.TEAMSPEAK3_SERVER_IP, settings.TEAMSPEAK3_SERVER_PORT)
        server.login(settings.TEAMSPEAK3_SERVERQUERY_USER, settings.TEAMSPEAK3_SERVERQUERY_PASSWORD)
        server.use(settings.TEAMSPEAK3_VIRTUAL_SERVER)
        logger.debug("Got TS3 server instance based on settings.")
        return server

    @staticmethod
    def __santatize_username(username):
        sanatized = username.replace(" ", "_")
        sanatized = sanatized.replace("'", "-")
        return sanatized

    def _get_userid(self, uid):
        logger.debug("Looking for uid %s on TS3 server." % uid)
        try:
            ret = self.server.send_command('customsearch', {'ident': 'sso_uid', 'pattern': uid})
            if ret and 'keys' in ret and 'cldbid' in ret['keys']:
                logger.debug("Got userid %s for uid %s" % (ret['keys']['cldbid'], uid))
                return ret['keys']['cldbid']
        except TeamspeakError as e:
            if not e.code == '1281':
                raise e
        return None

    def _group_id_by_name(self, groupname):
        logger.debug("Looking for group %s on TS3 server." % groupname)
        group_cache = self.server.send_command('servergrouplist')
        logger.debug("Received group cache from server: %s" % group_cache)
        for group in group_cache:
            logger.debug("Checking group %s" % group)
            if group['keys']['name'] == groupname:
                logger.debug("Found group %s, returning id %s" % (groupname, group['keys']['sgid']))
                return group['keys']['sgid']
        logger.debug("Group %s not found on server." % groupname)
        return None

    def _create_group(self, groupname):
        logger.debug("Creating group %s on TS3 server." % groupname)
        sgid = self._group_id_by_name(groupname)
        if not sgid:
            logger.debug("Group does not yet exist. Proceeding with creation.")
            ret = self.server.send_command('servergroupadd', {'name': groupname})
            self.__group_cache = None
            sgid = ret['keys']['sgid']
            self.server.send_command('servergroupaddperm',
                                     {'sgid': sgid, 'permsid': 'i_group_needed_modify_power', 'permvalue': 75,
                                      'permnegated': 0, 'permskip': 0})
            self.server.send_command('servergroupaddperm',
                                     {'sgid': sgid, 'permsid': 'i_group_needed_member_add_power', 'permvalue': 100,
                                      'permnegated': 0, 'permskip': 0})
            self.server.send_command('servergroupaddperm',
                                     {'sgid': sgid, 'permsid': 'i_group_needed_member_remove_power', 'permvalue': 100,
                                      'permnegated': 0, 'permskip': 0})
        logger.info("Created group on TS3 server with name %s and id %s" % (groupname, sgid))
        return sgid

    def _user_group_list(self, cldbid):
        logger.debug("Retrieving group list for user with id %s" % cldbid)
        server = Teamspeak3Manager.__get_created_server()
        try:
            groups = self.server.send_command('servergroupsbyclientid', {'cldbid': cldbid})
        except TeamspeakError as e:
            if e.code == '1281': # no groups
                groups = []
            else:
                raise e
        logger.debug("Retrieved group list: %s" % groups)
        outlist = {}

        if type(groups) == list:
            logger.debug("Recieved multiple groups. Iterating.")
            for group in groups:
                outlist[group['keys']['name']] = group['keys']['sgid']
        elif type(groups) == dict:
            logger.debug("Recieved single group.")
            outlist[groups['keys']['name']] = groups['keys']['sgid']
        logger.debug("Returning name/id pairing: %s" % outlist)
        return outlist

    def _group_list(self):
        logger.debug("Retrieving group list on TS3 server.")
        group_cache = self.server.send_command('servergrouplist')
        logger.debug("Received group cache from server: %s" % group_cache)
        outlist = {}
        if group_cache:
            for group in group_cache:
                logger.debug("Assigning name/id dict: %s = %s" % (group['keys']['name'], group['keys']['sgid']))
                outlist[group['keys']['name']] = group['keys']['sgid']
        else:
            logger.error("Received empty group cache while retrieving group cache from TS3 server. 1024 error.")
        logger.debug("Returning name/id pairing: %s" % outlist)
        return outlist

    def _add_user_to_group(self, uid, groupid):
        logger.debug("Adding group id %s to TS3 user id %s" % (groupid, uid))
        user_groups = self._user_group_list(uid)

        if groupid not in user_groups.values():
            logger.debug("User does not have group already. Issuing command to add.")
            self.server.send_command('servergroupaddclient',
                                     {'sgid': str(groupid), 'cldbid': uid})
            logger.info("Added user id %s to group id %s on TS3 server." % (uid, groupid))

    def _remove_user_from_group(self, uid, groupid):
        logger.debug("Removing group id %s from TS3 user id %s" % (groupid, uid))
        user_groups = self._user_group_list(uid)

        if str(groupid) in user_groups.values():
            logger.debug("User is in group. Issuing command to remove.")
            self.server.send_command('servergroupdelclient',
                                     {'sgid': str(groupid), 'cldbid': uid})
            logger.info("Removed user id %s from group id %s on TS3 server." % (uid, groupid))

    def _sync_ts_group_db(self):
        logger.debug("_sync_ts_group_db function called.")
        try:
            remote_groups = self._group_list()
            local_groups = TSgroup.objects.all()
            logger.debug("Comparing remote groups to TSgroup objects: %s" % local_groups)
            for key in remote_groups:
                logger.debug("Typecasting remote_group value at position %s to int: %s" % (key, remote_groups[key]))
                remote_groups[key] = int(remote_groups[key])

            for group in local_groups:
                logger.debug("Checking local group %s" % group)
                if group.ts_group_id not in remote_groups.values():
                    logger.debug(
                        "Local group id %s not found on server. Deleting model %s" % (group.ts_group_id, group))
                    TSgroup.objects.filter(ts_group_id=group.ts_group_id).delete()
            for key in remote_groups:
                g = TSgroup(ts_group_id=remote_groups[key], ts_group_name=key)
                q = TSgroup.objects.filter(ts_group_id=g.ts_group_id)
                if not q:
                    logger.debug("Local group does not exist for TS group %s. Creating TSgroup model %s" % (
                        remote_groups[key], g))
                    g.save()
        except TeamspeakError as e:
            logger.error("Error occured while syncing TS group db: %s" % str(e))
        except:
            logger.exception("An unhandled exception has occured while syncing TS groups.")

    def add_user(self, user, fmt_name):
        username_clean = self.__santatize_username(fmt_name[:30])
        logger.debug("Adding user to TS3 server with cleaned username %s" % username_clean)
        server_groups = self._group_list()

        state = user.profile.state.name
        if state not in server_groups:
             self._create_group(state)

        state_group_id = self._group_id_by_name(state)

        try:
            ret = self.server.send_command('tokenadd', {'tokentype': 0, 'tokenid1': state_group_id, 'tokenid2': 0,
                                                        'tokendescription': username_clean,
                                                        'tokencustomset': "ident=sso_uid value=%s" % username_clean})
        except TeamspeakError as e:
            logger.error("Failed to add teamspeak user %s: %s" % (username, str(e)))
            return "",""

        try:
            token = ret['keys']['token']
            logger.info("Created permission token for user %s on TS3 server" % username_clean)
            return username_clean, token
        except:
            logger.exception("Failed to add teamspeak user %s - received response: %s" % (username_clean, ret))
            return "", ""

    def delete_user(self, uid):
        user = self._get_userid(uid)
        logger.debug("Deleting user %s with id %s from TS3 server." % (user, uid))
        if user:
            clients = self.server.send_command('clientlist')
            if isinstance(clients, dict):
                # Rewrap list
                clients = [clients]
                
            for client in clients:
                try:
                    if client['keys']['client_database_id'] == user:
                        logger.debug("Found user %s on TS3 server - issuing deletion command." % user)
                        self.server.send_command('clientkick', {'clid': client['keys']['clid'], 'reasonid': 5,
                                                                'reasonmsg': 'Auth service deleted'})
                except:
                    logger.exception("Failed to delete user id %s from TS3 - received response %s" % (uid, client))
                    return False
            try:
                ret = self.server.send_command('clientdbdelete', {'cldbid': user})
            except TeamspeakError as e:
                logger.error("Failed to delete teamspeak user %s: %s" % (uid, str(e)))
                return False
            if ret == '0':
                logger.info("Deleted user with id %s from TS3 server." % uid)
                return True
            else:
                logger.exception("Failed to delete user id %s from TS3 - received response %s" % (uid, ret))
                return False
        else:
            logger.warn("User with id %s not found on TS3 server. Assuming succesful deletion." % uid)
            return True

    def check_user_exists(self, uid):
        if self._get_userid(uid):
            return True

        return False

    def generate_new_permissionkey(self, uid, user, username):
        logger.debug("Re-issuing permission key for user id %s" % uid)
        self.delete_user(uid)
        return self.add_user(user, username)

    def update_groups(self, uid, ts_groups):
        logger.debug("Updating uid %s TS3 groups %s" % (uid, ts_groups))
        userid = self._get_userid(uid)
        addgroups = []
        remgroups = []
        if userid is not None:
            user_ts_groups = self._user_group_list(userid)
            logger.debug("User has groups on TS3 server: %s" % user_ts_groups)
            for key in user_ts_groups:
                user_ts_groups[key] = int(user_ts_groups[key])
            for ts_group_key in ts_groups:
                logger.debug("Checking if user has group %s on TS3 server." % ts_group_key)
                if ts_groups[ts_group_key] not in user_ts_groups.values():
                    addgroups.append(ts_groups[ts_group_key])
            for user_ts_group_key in user_ts_groups:
                if user_ts_groups[user_ts_group_key] not in ts_groups.values():
                    remgroups.append(user_ts_groups[user_ts_group_key])

            for g in addgroups:
                logger.info("Adding Teamspeak user %s into group %s" % (userid, g))
                self._add_user_to_group(userid, g)

            for g in remgroups:
                logger.info("Removing Teamspeak user %s from group %s" % (userid, g))
                self._remove_user_from_group(userid, g)



from django.conf.urls import url

from . import views

app_name = 'timerboard'

urlpatterns = [
    url(r'^$', views.TimerView.as_view(), name='view'),
    url(r'^add/$', views.AddTimerView.as_view(), name='add'),
    url(r'^remove/(?P<pk>\w+)$', views.RemoveTimerView.as_view(), name='delete'),
    url(r'^edit/(?P<pk>\w+)$', views.EditTimerView.as_view(), name='edit'),
]
